//
// "$Id: $"
//
//
//                   Copyright (c) 2003-2004, O'ksi'D
//                      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of O'ksi'D nor the names of its contributors
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//

#include "Clock.h"
#include "DevIo.h"
#include "Gui.h"
#include <stdlib.h>
#include <stdio.h>
#include <FL/Fl.H>

#ifdef WIN32
#include <winsock.h>
#else
#include <sys/time.h>
#endif


#ifdef WIN32
#include <windows.h>
#include <io.h>
#include <mmsystem.h>
#include <time.h>
#define usleep(x) Sleep((x) / 1000)
int gettimeofday(struct timeval *tp, void *tzp);
#else
#include <unistd.h>
#endif
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

extern Gui *gui;

Clock::Clock()
{

}


#ifdef WIN32
int gettimeofday(struct timeval *tp, void *tzp)
{
	DWORD tx;
	static int init = 0;
	static timeval tv;
	static DWORD lt;
	DWORD dt;
	if (!init) {
		SYSTEMTIME st;
		time_t ti;
		struct tm tk;
 
		GetLocalTime (&st);
		lt = timeGetTime();
		tk.tm_sec = st.wSecond;
		tk.tm_min = st.wMinute;
		tk.tm_hour = st.wHour;
		tk.tm_mday = st.wDay;
		tk.tm_mon = st.wMonth - 1;
		tk.tm_year = st.wYear - 1900;
		tk.tm_isdst = -1;
		ti = mktime (&tk);
		tv.tv_sec = ti;
		tv.tv_usec = 0;
		init = 1;
	}
	//timeBeginPeriod(1);
	tx = timeGetTime();
	//timeEndPeriod(1);
	
	dt = tx - lt;
	//printf("%ld\n", dt);
	{
		DWORD t;
		t = dt / 1000;
		tp->tv_sec = tv.tv_sec + t;
		dt -= t * 1000;
	}
	tp->tv_usec = tv.tv_usec + dt * 1000;
	if (tp->tv_usec >= 1000000) {
		tp->tv_usec -= 1000000;
		tp->tv_sec += 1;	
	}	
	return 0;
}
#endif


//
// Swap the buffers and create a synchro point  for IO threads at 44Hz
//
void *clock_thread(void *data)
{
	int t = 0;
	int t0;
	struct timeval tv, tm, tva;
	Gui *gui = (Gui*) data;
	
	gettimeofday(&tm, NULL);
	gettimeofday(&tva, NULL);
	t0 = tm.tv_sec;

	while (1) {
		int i;

		// let the sequencers do their jobs !
		t = (tva.tv_sec - t0) * 100 + tva.tv_usec / 10000;
		Gui::lock();
		gui->clock_edit->it_s_time_to_do_it(t);
		//gui->list_edit->it_s_time_to_do_it(t);
		gui->cue_edit->it_s_time_to_do_it(t);
		gui->sequence->it_s_time_to_do_it(t);
		Gui::unlock();

		// lock the IO threads !
		DevIo::lock_all();
		
		for (i = 0; i < 32; i++) {
			gui->in_universes[i]->swap_buffer();
			gui->out_universes[i]->swap_buffer();
		}

		// now the io threads can do their job !
		DevIo::rid++;
		DevIo::unlock_all();

		// precise delay 

		do {
			tm.tv_usec += gui->universes->timeout.tv_usec;
			if (tm.tv_usec >= 1000000) {
				tm.tv_usec -= 1000000;
				tm.tv_sec++;
			}
			//tm.tv_sec += gui->universes->timeout.tv_sec;
			gettimeofday(&tva, NULL);
			tv.tv_sec = tm.tv_sec - tva.tv_sec;
			tv.tv_usec = tm.tv_usec - tva.tv_usec;
			if (tv.tv_usec < 0) {
				tv.tv_usec += 1000000;
				tv.tv_sec--;
			}
			//if (!(tv.tv_sec >= 0 && tv.tv_usec >= 0)) {
			//	gui->jump = 30;
			//}
		} while (!(tv.tv_sec >= 0 && tv.tv_usec >= 0) && (gui->jump = 30));

		// sleep as needed.
#ifdef WIN32	
		SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_HIGHEST);
		Sleep(tv.tv_usec / 1000);
		SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_NORMAL);
#else
		select(0, NULL, NULL, NULL, &tv);
#endif

		
		

		if (gui->jump > 0) gui->jump--;
	}
	return 0;
}


//
// End of "$Id: $".
//

