//
// "$Id: $"
//
//
//                   Copyright (c) 2003, O'ksi'D
//                      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of O'ksi'D nor the names of its contributors
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//

#include "CueEdit.h"
#include "Gui.h"
#include <FL/Fl_File_Chooser.H>

#ifdef WIN32
#define snprintf _snprintf
#endif

extern Gui *gui;

static void cue_sel_but_cb(Fl_Widget *w, void *d)
{
	((CueEdit*) d)->open_cue(NULL);
}
 
static void cue_save_but_cb(Fl_Widget *w, void *d)
{
	((CueEdit*) d)->save_cue(((CueEdit*) d)->data->file_name);
}

static void cue_saveas_but_cb(Fl_Widget *w, void *d)
{
	((CueEdit*) d)->save_cue(NULL);
}

static void cue_new_but_cb(Fl_Widget *w, void *d)
{
	((CueEdit*) d)->new_cue();
}

static void cue_step_minus_cb(Fl_Widget *w, void *d)
{
	char buf[256];
	CueEdit *s = (CueEdit*) d; 
	s->step -= 100 + s->step % 100;
	if (s->step < 0) s->step = 0;
	if (s->step >= s->data->time_length) {
		s->step = 0;
	}
	snprintf(buf, 256, "%05d.%02d", s->step / 100, s->step % 100);
	s->cue_step->value(buf); 

	s->need_sync_output = 1;
}

static void cue_step_plus_cb(Fl_Widget *w, void *d)
{
	char buf[256];
	CueEdit *s = (CueEdit*) d; 
	s->step += 100 - s->step % 100;
	if (s->step < 0) s->step = 0;
	if (s->step >= s->data->time_length) {
		s->step = s->data->time_length - 100;
	}
	snprintf(buf, 256, "%05d.%02d", s->step / 100, s->step % 100);
	s->cue_step->value(buf); 
	s->need_sync_output = 1;
}

static void cue_step_cb(Fl_Widget *w, void *d)
{
	char buf[256];
	CueEdit *s = (CueEdit*) d; 
	s->step = (int)(atof(s->cue_step->value()) * 100.0);
	s->step -= s->step % 100;
	if (s->step < 0) s->step = 0;
	if (s->step >= s->data->time_length) {
		s->step = s->data->time_length - 100;
	}

	snprintf(buf, 256, "%05d.%02d", s->step / 100, s->step % 100);
	s->cue_step->value(buf); 

	s->need_sync_output = 1;
}

static void cue_begin_cb(Fl_Widget *w, void *d)
{
	char buf[256];
	CueEdit *s = (CueEdit*) d; 
	s->step = 0;
	snprintf(buf, 256, "%05d.%02d", s->step / 100, s->step % 100);
	s->cue_step->value(buf); 
	s->need_sync_output = 1;
	s->start_10ms = s->last_10ms + 1;

}

static void cue_end_cb(Fl_Widget *w, void *d)
{
	char buf[256];
	CueEdit *s = (CueEdit*) d; 
	

	s->step = s->data->time_length - 100;
	if (s->step < 0) s->step = 0;
	snprintf(buf, 256, "%05d.%02d", s->step / 100, s->step % 100);
	s->cue_step->value(buf); 

	s->start_10ms = s->last_10ms + 1 - s->data->time_length;
	s->need_sync_output = 1;
}

static void cue_stop_cb(Fl_Widget *w, void *d)
{
	CueEdit *s = (CueEdit*) d; 

	s->cue_play->value(0);
	s->play = 0;
}

static void cue_play_cb(Fl_Widget *w, void *d)
{
	CueEdit *s = (CueEdit*) d; 

	s->start_10ms = s->last_10ms + 1 - s->step;
	s->cue_play->value(1);
	s->play = 1;
}

static void cue_bpm_cb(Fl_Widget *w, void *d)
{
	char buf[256];
	CueEdit *s = (CueEdit*) d; 
	
	s->data->bpm = (int)(atof(s->cue_bpm->value()) * 100.0);
	if (s->data->bpm< 0) s->data->bpm = 0;
	snprintf(buf, 256, "%d.%02d", s->data->bpm / 100, s->data->bpm % 100);
	s->cue_bpm->value(buf); 
}

static void cue_live_cb(Fl_Widget *w, void *d)
{
	CueEdit *s = (CueEdit*) d; 
	
	s->data->live = s->cue_live->value();
}

static void cue_loop_cb(Fl_Widget *w, void *d)
{
	//CueEdit *s = (CueEdit*) d; 
	
}

static void cue_export_cb(Fl_Widget *w, void *d)
{
	//CueEdit *s = (CueEdit*) d; 
	
	// TODO
}



CueEdit::CueEdit(int X, int Y, int W, int H, const char *T) :
	Fl_Group(X, Y, W, H, T)
{
	step = 0;
	data = new Cue();

	need_reload = 0;
	cue_pool = NULL;
	ma_pool = 0;
	nb_pool = 0;

	last_10ms = 0;
	
	play = 0;

	cue_file_inp = new Fl_Input(X + 5, Y + 4, 400, 25);
	cue_file_inp->value("default.cue");

	cue_new_but = new Fl_Button(X + 410, Y + 4, 40, 25, "new");
	cue_new_but->callback(cue_new_but_cb, this);
	cue_sel_but = new Fl_Button(X + 455, Y + 4, 40, 25, "open");
	cue_sel_but->callback(cue_sel_but_cb, this);
	cue_save_but = new Fl_Button(X + 500, Y + 4, 40, 25, "save");
	cue_save_but->callback(cue_save_but_cb, this);
	cue_saveas_but = new Fl_Button(X + 545, Y + 4, 60, 25, "save as");
	cue_saveas_but->callback(cue_saveas_but_cb, this);


	cue_step = new Fl_Float_Input(X + 35, Y + 32, 80, 25, "Step");
	cue_step->callback(cue_step_cb, this);
	cue_step->when(FL_WHEN_ENTER_KEY | FL_WHEN_RELEASE );
	cue_step->value("00000.00");
	
	cue_step_plus = new Fl_Button(X + 116, Y + 32, 12, 12, "+");
	cue_step_plus->callback(cue_step_plus_cb, this);
	cue_step_minus = new Fl_Button(X + 116, Y + 43, 12, 12, "-");
	cue_step_minus->callback(cue_step_minus_cb, this);

	cue_length = new Fl_Output(X + 178, Y + 32, 80, 25, "Length");
	cue_length->color(FL_GRAY);
	cue_length->value("0.00");

	cue_bpm = new Fl_Float_Input(X + 292, Y + 32, 50, 25, "BPM");
	cue_bpm->when(FL_WHEN_ENTER_KEY | FL_WHEN_RELEASE );
	cue_bpm->callback(cue_bpm_cb, this);
	cue_bpm->value("60.00");

	cue_live = new Fl_Button(X + 350, Y + 32, 25, 25, "Li");
	cue_live->type(FL_TOGGLE_BUTTON);
	cue_live->callback(cue_live_cb, this);
	cue_live->value(1);

	cue_loop = new Fl_Button(X + 380, Y + 32, 25, 25, "Lo");
	cue_loop->type(FL_TOGGLE_BUTTON);
	cue_loop->value(1);
	cue_loop->callback(cue_loop_cb, this);

	cue_begin = new Fl_Button(X + 410, Y + 32, 25, 25, "<<");
	cue_begin->callback(cue_begin_cb, this);

	cue_end = new Fl_Button(X + 438, Y + 32, 25, 25, ">>");
	cue_end->callback(cue_end_cb, this);

	cue_play = new Fl_Button(X + 470, Y + 32, 25, 25, "Pl");
	//cue_play->type(FL_TOGGLE_BUTTON);
	cue_play->callback(cue_play_cb, this);
	cue_play->when(FL_WHEN_RELEASE);
	cue_play->value(0);

	cue_stop = new Fl_Button(X + 498, Y + 32, 25, 25, "St");
	cue_stop->callback(cue_stop_cb, this);

	cue_export = new Fl_Button(X + 530, Y + 32, 50, 25, "Export");
	cue_export->callback(cue_export_cb, this);

	cue_table = new CueTable(X, Y + 60, W, H - 60, this);


	res = new Fl_Box( X + W - 2, Y + H - 2, 2, 2);
	res->hide();
	resizable(res);

	hide();
	end();
}


void CueEdit::save_cue(const char *file)
{
	char buf[2048];
	int dl;

	snprintf(buf, 2047, "%s/cue/", gui->work_dir);
	dl = strlen(buf);
	fl_make_path(buf);
	
	
	if (!file) 
	{
	
		snprintf(buf, 2047, "%s/cue/%s", gui->work_dir,
				cue_file_inp->value());

		Fl_File_Chooser fc(buf, 
			"Cue (*.cue)", Fl_File_Chooser::CREATE, 
			"Save as cue ...");
		fc.show();
		while (fc.visible())
			Fl::wait();
	
		if (fc.count() > 0 && fc.value(0) && !strncmp(fc.value(0), buf, dl)) {
			cue_file_inp->value(fc.value(0) + dl);
		} else {
			if (fc.count() > 0) fl_alert("Unable to save to file!");
			return;
		} 
	} else {
		if (!strncmp(file, buf, dl)) {
			cue_file_inp->value(file + dl);
		} else {
			fl_alert("Unable to save file!");
			return;
		}	
	}

	dl = strlen(cue_file_inp->value());
	if (dl > 3 && !strcmp(cue_file_inp->value() + dl - 4, ".cue")) {
		snprintf(buf, 2047, "%s/cue/%s", gui->work_dir, 
			cue_file_inp->value());
	} else {
		snprintf(buf, 2047, "%s/cue/%s.cue", gui->work_dir, 
			cue_file_inp->value());
	}
	fl_make_path_for_file(buf);	
	if (data->save(buf)) fl_alert("Unable to save file!");
	need_reload = 1;
	get(buf);
}


void CueEdit::new_cue()
{
	play = 0;
	cue_play->value(0);
	if (data->changed) {
		if (fl_ask("Save changes ?")) save_cue(data->file_name);
	}
	redraw();
	Gui::lock();
	delete (data);
	data = new Cue();
	Gui::unlock();

	cue_file_inp->value("default.cue");
	need_sync_output = 1;
	step = 0;
	cue_step->value("00000.00");
	cue_loop->value(1);
	cue_length->value("0.00");
	cue_bpm->value("60.00"); 

	redraw();
}

void CueEdit::open_cue(const char *file)
{
	char buf[2048];
	int dl;

	if (data->changed) {
		if (fl_ask("Save changes ?")) save_cue();
	}	

	snprintf(buf, 2047, "%s/cue/", gui->work_dir);
	dl = strlen(buf);
	fl_make_path(buf);


	if (!file) {
		Fl_File_Chooser fc(buf, 
			"Cue (*.cue)", Fl_File_Chooser::CREATE, 
			"Open cue ...");
		fc.show();
		while (fc.visible())
			Fl::wait();
	
		if (fc.count() > 0 && fc.value(0) && !strncmp(fc.value(0), buf, dl)) {
			cue_file_inp->value(fc.value(0) + dl);		
		} else {
			if (fc.count() > 0) fl_alert("Unable to open file!");
			return;
		} 
	} else {
		if (!strncmp(file, buf, dl)) {
			cue_file_inp->value(file + dl);
		} else {
			fl_alert("Unable to open file!");
			return;
		}	
	}

	Gui::lock();
	delete (data);
	data = new Cue();
	Gui::unlock();

	snprintf(buf, 2047, "%s/cue/%s", gui->work_dir, cue_file_inp->value());
	
	if (data->load(buf)) {
		fl_alert("Unable to open file!");
	}

	step = 0;
	cue_step->value("00000.00");
	cue_loop->value(1);
	snprintf(buf, 24, "%d.%02d", data->time_length / 100,
		data->time_length % 100);

	cue_length->value(buf);
	snprintf(buf, 256, "%d.%02d", data->bpm / 100, data->bpm % 100);
	cue_bpm->value(buf); 

	cue_table->rows(data->nb_seq + 8);

	need_sync_output = 1;	
	redraw();
}

void CueEdit::it_s_time_to_do_it(int time_10ms)
{
	last_10ms = time_10ms;

	if (play) {
		char buf[256];
		step = (int)((time_10ms - start_10ms) * (data->bpm / 6000.0));	
		
		if (step >= data->time_length) {
			//int n;
			if (!cue_loop->value()) {
				cue_play->value(0);
				play = 0;
				Fl::awake();
				return;
			}
			//n = (int)(data->time_length / (data->bpm / 6000.0));
			//if (n < 1) n = 1;
			//start_10ms += n; 
			//step = (int)((time_10ms - start_10ms) * (data->bpm / 6000.0));
		}
		if (step < 0) step = 0;
		snprintf(buf, 256, "%05d.%02d", step / 100, step % 100);
		cue_step->value(buf);
		Fl::awake();
		if (need_sync_output) need_sync_output = 0;
		data->do_step(step);
	} else 	if (need_sync_output) {
		need_sync_output = 0;
		data->do_step(step);
		Fl::awake();
	}
	
}



int CueEdit::handle(int e)
{
	if (e == FL_SHORTCUT && Fl::event_length() == 1) {
		switch (Fl::event_text()[0]) {
		case '+':
			cue_step_plus->do_callback();
			return 1;
		case '-':
			cue_step_minus->do_callback();
			return 1;
		case ' ':
			if (play) {
				cue_stop->do_callback();
			} else {
				cue_play->do_callback();
			}
			return 1;
		}	
	}
	return Fl_Group::handle(e);
}

Cue *CueEdit::get(const char *seq)
{
	int i, f;
	f = -1;
	for (i = 0; i < nb_pool; i++) {
		if (cue_pool[i]) {
			if (cue_pool[i]->file_name &&
				!strcmp(seq, cue_pool[i]->file_name)) 
			{
				if (need_reload) cue_pool[i]->load(seq);
				need_reload = 0;
				return cue_pool[i];
			}
		} else {
			f = i;
		}
	}

	if (f == -1) {
		f = nb_pool++;
		if (f >= ma_pool) {
			int i;
			ma_pool += 32;
			cue_pool = (Cue**) realloc(cue_pool,
				sizeof(Cue*) * (ma_pool));
			for (i = f; i < ma_pool; i++) {
				cue_pool[i] = NULL;
			}
		}
	}

	cue_pool[f] = new Cue();
	cue_pool[f]->load(seq);
	return cue_pool[f];
}

//
// End of "$Id: $".
//

