//
// "$Id: $"
//
//
//                   Copyright (c) 2003-2004, O'ksi'D
//                      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of O'ksi'D nor the names of its contributors
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//

#include "CueEdit.h"
#include "Gui.h"
#include "sexpr.h"
#include <FL/Fl_Window.H>
#include <FL/Fl.H>
#include <FL/fl_ask.H>
#include <FL/fl_draw.H>
#include <FL/Fl_Preferences.H>
#include <stdlib.h>
#include <stdio.h>
#ifdef WIN32
#define snprintf _snprintf
#define S_IRGRP 0
#define S_IROTH 0
#include <io.h>
#else
#include <unistd.h>
#endif
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <FL/Fl_File_Chooser.H>

extern Gui *gui;

CueTable::CueTable(int x, int y, int w, int h, CueEdit *d) : 
	Fl_Table(x, y, w, h)
{
	cue_edit = d;

	callback(&event_callback, (void*)this);
	when(FL_WHEN_NOT_CHANGED|when()|FL_WHEN_RELEASE);

	(new Fl_Box(9999,9999,0,0))->hide();  // HACK: prevent flickering in Fl_Scroll

	cols(3);
	rows(11);
	col_header(1);
	col_resize(1);
	row_resize(1);
	row_height_all(25);

	col_header_height(30);
	col_width(0, 30);
	col_width(1, 400);
	
	do_edit = 0;

	end();
}

void CueTable::set_value()
{
	char buf[2048];
	int dl;

	if (row_edit >= rows() - 2) rows(rows() + 8);
	if (col_edit == 0) {
		cue_edit->data->replace(row_edit, NULL);
	} else if (col_edit == 1) {
		snprintf(buf, 2047, "%s/sequence/", gui->work_dir);
		dl = strlen(buf);
		fl_make_path(buf);

		Fl_File_Chooser fc(buf, 
			"Sequence (*.seq)", Fl_File_Chooser::CREATE, 
			"Open sequence ...");
		fc.show();
		while (fc.visible())
			Fl::wait();
	
		if (fc.count() > 0 && fc.value(0) && !strncmp(fc.value(0), buf, dl)) {
			cue_edit->data->replace(row_edit, fc.value(0) + dl);		
		} else {
			if (fc.count() > 0) fl_alert("Unable to open file!");
			return;
		} 
	}
	snprintf(buf, 24, "%d.%02d", cue_edit->data->time_length / 100,
		cue_edit->data->time_length % 100);
	cue_edit->cue_length->value(buf);

	redraw();
}

// Callback whenever someone clicks on different parts of the table
void CueTable::event_callback(Fl_Widget*, void *data)
{
    CueTable *o = (CueTable*)data;
    o->event_callback2();
}

void CueTable::event_callback2()
{
    int R = callback_row(),
        C = callback_col();
    TableContext context = callback_context();



    switch ( context )
    {
	case CONTEXT_CELL:
	{
            switch (Fl::event()) 
	    { 
		case FL_RELEASE:
		    do_edit = 0;
		    if (row_edit == R && col_edit == C) {
			set_value();
		    }
		    set_selection(R, C, R, C);
		    redraw();
		    return;
		case FL_PUSH:
		    set_selection(R, C, R, C);
		    do_edit = 1;
		    row_edit = R;
		    col_edit = C;
		    Fl::event_clicks(0);
		    return;
		case FL_KEYBOARD:
		    if (Fl::event() == FL_KEYBOARD && Fl::e_length == 0)
		        { return; }
	
		    if (R > rows() - 2) rows(rows() + 8);
		    row_edit = R;
		    col_edit = C;

		    set_selection(R, C, R, C);
		    if (Fl::event_length() == 1 && 
			Fl::event_text()[0] == '\r') 
		    {
			set_value();
		    }
		   
		    return;
	    }
	    return;
	}

	case CONTEXT_ROW_HEADER:
	case CONTEXT_COL_HEADER:
	    do_cb();
	    hide_inp();
	    return;

	case CONTEXT_TABLE:
	    if (R < 0 && C < 0)
	    {
		do_cb();
		hide_inp();
	    }
	    return;
	
	default:
	    return;
    }
}

void CueTable::do_cb()
{
	//if (input->visible()) input->do_callback();

}

void CueTable::hide_inp()
{
//	input->hide();
}

// Handle drawing all cells in table
void CueTable::draw_cell(TableContext context, 
			    int R, int C, int X, int Y, int W, int H)
{
    static char s[30];

    switch ( context )
    {
	case CONTEXT_STARTPAGE:
	    // Get kb nav + mouse 'selection region' for use below
	    get_selection(s_top, s_left, s_bottom, s_right);	
	    break;

	case CONTEXT_COL_HEADER:
	    fl_font(FL_HELVETICA | FL_BOLD, 14);
	    fl_push_clip(X, Y, W, H);
	    {
		fl_draw_box(FL_THIN_UP_BOX, X, Y, W, H, col_header_color());
		fl_color(FL_BLACK);
		switch (C) {
		case 0:
			fl_draw("Del", X, Y, W, H, FL_ALIGN_CENTER);
			break;
		case 1:
			fl_draw("Sequence name", X, Y, W, H, FL_ALIGN_CENTER);
			break;
		case 2:
			fl_draw("Length", X, Y, W, H, FL_ALIGN_CENTER);
			break;
		case 3:
			fl_draw("Loop", X, Y, W, H, FL_ALIGN_CENTER);
			break;
		case 4:
			fl_draw("Clock\nsource", X, Y, W, H, FL_ALIGN_CENTER);
			break;
		case 5:
			fl_draw("Clock\nmul/div", X, Y, W, H, FL_ALIGN_CENTER);
			break;

		}
	    }
	    fl_pop_clip();
	    return;

	case CONTEXT_ROW_HEADER:
	    fl_font(FL_HELVETICA | FL_BOLD, 14);
	    fl_push_clip(X, Y, W, H);
	    {
/*
		fl_draw_box(FL_THIN_UP_BOX, X, Y, W, H, row_header_color());
		fl_color(FL_BLACK);
		fl_draw(dev->get_value(R, C), X, Y, W, H, FL_ALIGN_LEFT);
*/
	    }
	    fl_pop_clip();
	    return;

	case CONTEXT_CELL:
	{

/*
	    if (R == row_edit && C == col_edit && 
		(input->visible()))
	    { 
		return; 
	    }
*/
	    // BACKGROUND
	    fl_push_clip(X, Y, W, H);

	    // Keyboard nav and mouse selection highlighting

	    if (R >= s_top && R <= s_bottom && C >= s_left && C <= s_right) {
		if (C == 0 || C == 1) {
		    if (row_edit == R && col_edit == C && do_edit) {
			fl_draw_box(FL_DOWN_BOX, X, Y, W, H, FL_GRAY);
		    } else {
			fl_draw_box(FL_UP_BOX, X, Y, W, H, FL_GRAY );
			fl_color(FL_YELLOW);
			fl_rect(X + 2, Y  + 2 , W - 4, H - 4);
		    }
		} else {
			fl_draw_box(FL_THIN_UP_BOX, X, Y, W, H, FL_YELLOW);
		}
            } else {
		if (C == 0 || C == 1) {
		    fl_draw_box(FL_UP_BOX, X, Y, W, H, FL_GRAY);
		} else {
	            fl_draw_box(FL_THIN_UP_BOX, X, Y, W, H, FL_WHITE); 
		}
	    }


 
	    fl_pop_clip();
	    // TEXT
	    fl_push_clip(X+3, Y+3, W-6, H-6);
	    {
		fl_font(FL_HELVETICA, 14);
		fl_color(FL_BLACK);
		if (C == 0) {
		    fl_draw("X", 
			X + 3, Y + 3, W - 6, H - 6, FL_ALIGN_LEFT);
		} else {
		    fl_draw(cue_edit->data->get_value(R, C), 
			X + 3, Y + 3, W - 6, H - 6, 
			(C == 1) ? FL_ALIGN_LEFT : FL_ALIGN_RIGHT);
		}
	    }
	    fl_pop_clip();
	    return;
	}

	case CONTEXT_RC_RESIZE:
	{

	    return;
	}
	default:
	    return;
    }
}


//
// End of "$Id: $".
//

