//
// "$Id: $"
//
//
//                   Copyright (c) 2003-2004, O'ksi'D
//                      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of O'ksi'D nor the names of its contributors
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//

#include "DevIo.h"


#include "Gui.h"
#include "okdmx31.h"

#include <stdlib.h>
#include <stdio.h>
#ifdef WIN32
#include <windows.h>
#include <io.h>
#include <mmsystem.h>
#include <time.h>
#else
#include <unistd.h>
#endif
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

extern Gui *gui;

void *devio_thread(void *data);

int DevIo::rid = 0;
DevIo *DevIo::inout[64] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
			0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
			0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
			0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
int DevIo::initialized = 0;
CRITICAL_SECTION DevIo::cs[64];

DevIo::DevIo(Gui *g, int i)
{
	id = i;
	gui = g;
	run = 0;
	ctrl_type = 0;
	ctrl_id = 0;
	thread = 0;
}

DevIo::~DevIo()
{
	if (run) stop_thread();
}

void DevIo::it_s_time_to_update(void)
{
	int i, ret;
	static Fl_Thread t = 0;


	if (thread != t) printf("XXXXXXX begin update\n");
	t = thread;
		
	// refresh IO monitor window
	refr++;
	if (refr > 20 || refr < 0) refr = 0;

	for (i = 0; i < 32; i++) {
		if (gui->out_universes[i]->ctrl_type->value() == ctrl_type &&
			(int)gui->out_universes[i]->ctrl_id->menu()[
				gui->out_universes[i]->ctrl_id->value()].user_data_ 
			== ctrl_id)
		{

			if (!refr) gui->out_universes[i]->refresh();

			ret = write(gui->out_universes[i]->buffer[
					!gui->out_universes[i]->current_buffer],
				gui->out_universes[i]->flags[
					!gui->out_universes[i]->current_buffer],
				gui->out_universes[i]->ctrl_universe->value());

			if (ret) {
				run = 0;
				gui->out_universes[i]->status->value(
					"Communication ERROR with controller !!!");
				gui->out_universes[i]->redraw();
					
			}
			//printf("%d\n",gui->out_universes[i]->buffer[
			//		gui->out_universes[i]->current_buffer][0]); 
				
		}
		
		if (gui->in_universes[i]->ctrl_type->value() == ctrl_type &&
			(int)gui->in_universes[i]->ctrl_id->menu()[
				gui->in_universes[i]->ctrl_id->value()].user_data_ 
			== ctrl_id)
		{
			ret = read(gui->in_universes[i]->buffer[
					!gui->in_universes[i]->current_buffer],
				gui->in_universes[i]->ctrl_universe->value());
			
			if (!refr) gui->in_universes[i]->refresh();

			if (ret) {
				run = 0;	
				gui->in_universes[i]->status->value(
					"Communication ERROR with controller !!!");
				gui->in_universes[i]->redraw();
				
			}
				
		}		
	}
}

int DevIo::write(const unsigned char *buf, 
			const unsigned char *flags, int univ)
{
	switch (ctrl_type) {
	case 1: // OksiD DMX 3/1
		return Okdmx31Write(ctrl_id, univ, buf); 
	}

	return 0;
}

int DevIo::read(unsigned char *buf, int univ)
{
	int ret;
	switch (ctrl_type) {
	case 1: // OksiD DMX 3/1
		ret = Okdmx31Read(ctrl_id, univ, buf);
		return ret; 
	}

	return 0;
}

int DevIo::has_device(int type, int id)
{
	int j;

	if (!type) return 1;

 	for (j = 0; j < 64; j++) {
		if (inout[j] && 
			type == inout[j]->ctrl_type &&
			id == inout[j]->ctrl_id)
		{
			return 1;	
		}
	}
	return 0;
}

void DevIo::update_controllers()
{
	Gui *gui = ::gui;
	int i, j;
	int used[64];

	Gui::lock();


	printf("begin update controller\n");
	for (j = 0; j < 64; j++) {
		used[j] = 0;
	}

	// find if the device is still used
	for (j = 0; j < 64; j++) {	
  	    for (i = 0; i < 32; i++) {
		if (inout[j] && 
			gui->out_universes[i]->ctrl_type->value() == inout[j]->ctrl_type &&
			(int)gui->out_universes[i]->ctrl_id->menu()[
				gui->out_universes[i]->ctrl_id->value()].user_data_ 
			== inout[j]->ctrl_id)
		{

			used[j] = 1;	
		}

		if (inout[j] && 
			gui->in_universes[i]->ctrl_type->value() == inout[j]->ctrl_type &&
			(int)gui->in_universes[i]->ctrl_id->menu()[
				gui->in_universes[i]->ctrl_id->value()].user_data_ 
			== inout[j]->ctrl_id)
		{
			used[j] = 1;
		}		
	    }
	}

	// delete it if not used
	for (j = 0; j < 64; j++) {
		if (!used[j] && inout[j]) {
			delete(inout[j]);
			inout[j] = NULL;
		}
	}

	printf("deleted update controller\n");


	// create device if not found
	for (i = 0; i < 32; i++) {
		int found;
		found = has_device(
				gui->in_universes[i]->ctrl_type->value(),
				(int)gui->in_universes[i]->ctrl_id->menu()[
					gui->in_universes[i]->ctrl_id->value()].user_data_);

		if (!found) {
			for (j = 0; j < 64; j++) {
				if (!inout[j]) {
					inout[j] = new DevIo(gui, j);
					inout[j]->create_thread(
						gui->in_universes[i]->ctrl_type->value(),
						(int)gui->in_universes[i]->ctrl_id->menu()[
							gui->in_universes[i]->ctrl_id->value()].user_data_); 	
					break;
				}
			}
		}

		found = has_device(
				gui->out_universes[i]->ctrl_type->value(),
				(int)gui->out_universes[i]->ctrl_id->menu()[
					gui->out_universes[i]->ctrl_id->value()].user_data_);

		if (!found) {
			for (j = 0; j < 64; j++) {
				if (!inout[j]) {
					inout[j] = new DevIo(gui, j);
					inout[j]->create_thread(
						gui->out_universes[i]->ctrl_type->value(),
						(int)gui->out_universes[i]->ctrl_id->menu()[
							gui->out_universes[i]->ctrl_id->value()].user_data_); 	
					break;
				}
			}
		}
	}

	printf("created update controller\n");

	// restart thread that had some problems...
	for (i = 0; i < 64; i++) {
		if (inout[i] && !inout[i]->run) {
			inout[i]->create_thread(inout[i]->ctrl_type,
					 inout[i]->ctrl_id);
		}
	} 
	printf("end update controller\n");

	Gui::unlock();

}

void DevIo::clear_controllers()
{

}

Fl_Menu_Item *DevIo::create_usb_menu(int type)
{
	return NULL;
}

void DevIo::create_thread(int t, int i)
{
	run = 1;
	ctrl_type = t;
	ctrl_id = i;
	fl_create_thread(thread, devio_thread, this);
}

void DevIo::stop_thread(void)
{
	run = 0;
	_cwait(NULL, thread, NULL);
	run = 0;
	thread = 0;
}

void DevIo::lock(void)
{
	if (!initialized) {
		int i;
		initialized = 1;
		for (i=0; i < 64; i++) {
#ifdef WIN32
			InitializeCriticalSection(&cs[i]);
#endif
		}
	}

#ifdef WIN32
	EnterCriticalSection(&cs[id]);
#endif
}

void DevIo::unlock(void)
{
#ifdef WIN32
	LeaveCriticalSection(&cs[id]);
#endif
}

void DevIo::lock_all(void)
{
	int i;
	if (!initialized) {
		initialized = 1;
		for (i=0; i < 64; i++) {
#ifdef WIN32
			InitializeCriticalSection(&cs[i]);
#endif
		}
	}

	for (i=0; i < 64; i++) {
#ifdef WIN32
		EnterCriticalSection(&cs[i]);
#endif
	}
}

void DevIo::unlock_all(void)
{
	int i;
	for (i=0; i < 64; i++) {
#ifdef WIN32
		LeaveCriticalSection(&cs[i]);
#endif
	}
}

//
// IO thread
//
void *devio_thread(void *data)
{
	DevIo *dev = (DevIo*) data;
	int last_id = DevIo::rid;
	while (dev->run) {
		// synchro point!
		dev->lock();
		if (last_id != DevIo::rid) {
			last_id = DevIo::rid;
			dev->it_s_time_to_update();
			dev->unlock();
		} else {
			dev->unlock();
#ifdef WIN32
			Sleep(10);
#endif
		}				
	}

	return 0;
}

/*
void Universe::write_device(long pos, unsigned char *buf, int len)
{
#ifdef WIN32
	unsigned char *ptr = buf;
	long n = (pos / 512);
	ptr -= pos - (n * 512);
	if (out_status[n]) {
		if (Okdmx31Write(out_port[n], out_channel[n], ptr)) {
			out_status[n] = 0;
		}
	}
#else
	lseek(fdo, pos, SEEK_SET);
	write(fdo, buf, len);
#endif
}

void Universe::read_device(long pos, unsigned char *buf)
{
#ifdef WIN32
	unsigned char *ptr = buf;
	long n = (pos / 512);
	ptr -= pos - (n * 512);
	if (in_status[n]) {
		if (Okdmx31Read(in_port[n], in_channel[n], ptr)) {
			in_status[n] = 0;
		}
	}
#else
	lseek(fdi, pos, SEEK_SET);
	read(fdi, buf, 512);
#endif
}

*/


//
// End of "$Id: $".
//

