//
// "$Id: $"
//
//
//                   Copyright (c) 2003, O'ksi'D
//                      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of O'ksi'D nor the names of its contributors
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//

#include "Device.h"
#include "Gui.h"
#include "sexpr.h"
#include <FL/Fl_Window.H>
#include <FL/Fl.H>
#include <FL/fl_ask.H>
#include <FL/Fl_Preferences.H>
#include <stdlib.h>
#include <stdio.h>
#ifdef WIN32
#define snprintf _snprintf
#define S_IRGRP 0
#define S_IROTH 0
#include <io.h>
#else
#include <unistd.h>
#endif
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <FL/Fl_File_Chooser.H>

static void map_sel_but_cb(Fl_Widget *w, void *d)
{
	Device *dev = (Device*) d;
	char *f = fl_file_chooser("Open DMX map", "*.dmap", dev->map_file);
	if (f) snprintf(dev->map_file, 1023, "%s", f);
	dev->load_map();
	dev->save_setup();
}

static void map_save_but_cb(Fl_Widget *w, void *d)
{
	Device *dev = (Device*) d;
	char *f = fl_file_chooser("Save DMX map", "*.dmap", dev->map_file);
	if (f) snprintf(dev->map_file, 1023, "%s", f);
	dev->save_map();
	dev->load_map();
	dev->save_setup();
}

static void map_new_but_cb(Fl_Widget *w, void *d)
{
	Device *dev = (Device*) d;
	if (dev->changed) map_save_but_cb(w, d);
	char *ptr = dev->map_file + strlen(dev->map_file);
	while (ptr > dev->map_file && *ptr != '/') ptr--;
	if (ptr > dev->map_file) *(ptr+1) = 0; 
	dev->clean_map();
	dev->map_inp->value(dev->map_file);
}

static void sel_tree_cb(Fl_Widget *w, void *d)
{
	Device *dev = (Device*) d;
	dev->tree_select();
}

static void name_inp_cb(Fl_Widget *w, void *d)
{
	Device *dev = (Device*) d;
	if (!dev->current_device) return;
	free(dev->current_device->name);
	dev->current_device->name = strdup(dev->name_inp->value());
	dev->sel_tree->redraw();
}

Device::Device(int X, int Y, int W, int H, const char *T) :
	Fl_Group(X, Y, W, H, T)
{

	map_inp = new Fl_Input(X + 35, Y + 5, 265, 20, "Map");
	map_sel_but = new Fl_Button(X + 305, Y + 5, 50, 20, "Select");
	map_sel_but->callback(map_sel_but_cb, this);
	map_save_but = new Fl_Button(X + 365, Y + 5, 40, 20, "Save");
	map_save_but->callback(map_save_but_cb, this);
	map_new_but = new Fl_Button(X + 415, Y + 5, 40, 20, "New");
	map_new_but->callback(map_new_but_cb, this);

	name_inp = new Fl_Input(X + 300, Y + 30, 305, 20);
	name_inp->callback(name_inp_cb, this);

	current_device = NULL;

	device_map.universe = -1;
	device_map.channel = -1;
	device_map.name = strdup("DMX Mapping");
	device_map.device = strdup("");
	device_map.child = NULL;
	device_map.nb_child = 0;

	sel_tree = new DeviceTree(X + 5, Y + 30, 290, H - 35);
	sel_tree->callback(sel_tree_cb, this);
	sel_tree->root(&device_map);

	load_setup();
	load_map();


	hide();
	end();
}

void Device::load_setup(void)
{
	char f[1024];
	char *ptr;

	changed = 0;

        Fl_Preferences app(Fl_Preferences::USER, "oksid.ch", "dmxstep");
        Fl_Preferences dev(app, "Device");
	snprintf(f,1023,  "%s/dmxstep/default.dmap", fl_getenv("HOME"));
	dev.get("map_file", map_file, f, 1023);
 	//uni.get("devo", devo, "/dev/dmx", 255);

	ptr = map_file;
	while (*ptr) {
		if (*ptr == '/') {
			*ptr = 0;
			if (fl_access(map_file, 0)) {
				fl_mkdir(map_file, S_IREAD|S_IWRITE|S_IEXEC|
					S_IRGRP|S_IROTH);
			}
			*ptr = '/';
		}
		ptr++;
	}
	 
}

void Device::save_setup(void)
{
        Fl_Preferences *app = new Fl_Preferences(Fl_Preferences::USER,
                                        "oksid.ch", "dmxstep");
        Fl_Preferences *dev = new Fl_Preferences(app, "Device");

	dev->set("map_file", map_file);

        delete(app);
        delete(dev);

	changed = 0;
 
}


void Device::load_map(void)
{
	map_inp->value(map_file);
}

static void fprint_dev_val(DeviceValue *dev, FILE *fd, int level)
{
	int i;
	static char spc[2048];
	static char buf[2048];

	if (level >= 2047) return;
	spc[level] = 0;
	if (level > 0) spc[level - 1] = ' ';
	fprintf(fd, "%s(%d %d ", spc, dev->universe, dev->channel);

	if (dev->name) {
		sexpr_quote(dev->name, strlen(dev->name), buf, 2048);
		fprintf(fd, "%s ", buf);
	} else {
		fprintf(fd, "\"\" ");
	}

	if (dev->name) {
		sexpr_quote(dev->device, strlen(dev->device), buf, 2048);
		fprintf(fd, "%s ", buf);
	} else {
		fprintf(fd, "\"\" ");
	}


	fprintf(fd, "\n");
	for (i = 0; i > dev->nb_child; i++) {
		fprint_dev_val(dev->child[i], fd, level + 1);
	}
	spc[level] = 0;
	fprintf(fd, "%s)\n", spc);
}

void Device::save_map(void)
{
	FILE *fd = fl_fopen(map_file, "w");
	if (!fd) {
		fl_alert("Unable to save DMX map !");
		return;
	}

	fprint_dev_val(&device_map, fd, 0);

	fclose(fd);
}

void Device::clean_map(DeviceValue *dev)
{
	if (!dev) dev = &device_map;

	for (int i = 0; i < dev->nb_child; i++) {
		clean_map(dev->child[i]);
		free(dev->child[i]);
	}
	dev->nb_child = 0;
	free(dev->child);
	dev->child = NULL;
	if (dev != &device_map) {
		free(dev->name);
		free(dev->device);
	}
	redraw();
}


void Device::tree_select()
{
	int n = sel_tree->get_selection();
	DeviceValue *dev = (DeviceValue*) sel_tree->item_data(n);
	if (!dev) return;

	current_device = dev;
	name_inp->value(dev->name);
}


//
// End of "$Id: $".
//

