//
// "$Id: $"
//
//
//                   Copyright (c) 2003, O'ksi'D
//                      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of O'ksi'D nor the names of its contributors
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//

#ifndef Device_H
#define Device_H

#include <FL/Fl_Group.H>
#include <FL/Fl_Choice.H>
#include <FL/Fl_Slider.H>
#include <FL/Fl_Scroll.H>
#include <FL/Fl_Box.H>
#include <FL/Fl_Pack.H>
#include <FL/Fl_Int_Input.H>
#include <FL/Fl_Toggle_Button.H>
#ifdef WIN32
#include <winsock.h>
#else
#include <sys/time.h>
#endif

#include "DeviceTree.h"
#include "stdlib.h"


struct DeviceValue {
	int universe;
	int channel;
	char *name;
	char *device;
	DeviceValue **child;
	int nb_child;
};

class Device : public Fl_Group {
public:
	char changed;
	char map_file[1024];
	Fl_Input *map_inp;
	Fl_Button *map_sel_but;
	Fl_Button *map_save_but;
	Fl_Button *map_new_but;

	DeviceTree *sel_tree;

	Fl_Input *name_inp;
	Fl_Input *dev_inp;
	Fl_Button *dev_sel_but;
	Fl_Int_Input *universe_inp;
	Fl_Int_Input *base_inp;
	Fl_Button *dev_del_but;
	Fl_Button *dev_new_but;

	Fl_Button *device_editor_but;

	DeviceValue device_map;
	DeviceValue *current_device;

	Device(int X, int Y, int W, int H, const char *t);
        void load_setup(void);
        void save_setup(void);
        void load_map(void);
        void save_map(void);
        void clean_map(DeviceValue *dev = NULL);
        void tree_select(void);

};


#endif // Device_H

//
// End of "$Id: $".
//

