//
// "$Id: $"
//
//
//                   Copyright (c) 2003, O'ksi'D
//                      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of O'ksi'D nor the names of its contributors
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//

#include "DeviceTree.h"
#include "Device.h"
#include <FL/fl_draw.H>
#include "Gui.h"
#include <stdlib.h>
#include <stdio.h>
#ifdef WIN32
#include <io.h>
#else
#include <unistd.h>
#endif
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

static void item_cb(Fl_Widget *w, void *d)
{
	((DeviceTree*)d)->item_event();
}

DeviceTree::DeviceTree(int X, int Y, int W, int H, const char* T) : 
	Fl_Tree (X, Y, W, H, T)
{
	callback(item_cb, this);
}

void DeviceTree::item_draw(int n, int X, int Y, int CX, int CY, int CW, int CH)
{
	DeviceValue *dev = (DeviceValue*) item_data(n);
	if (!dev) return;
	fl_font(0, 14);
	if (item_flags(n) & FLAG_SELECTED) {
		fl_color(FL_YELLOW);
		fl_rectf(X, Y, (int)fl_width(dev->name), fl_height());
	}
	fl_color(FL_BLACK);
	Y += fl_height() - fl_descent();
	fl_draw(dev->name, X, Y);
}

void DeviceTree::item_measure(int n)
{
	DeviceValue *dev = (DeviceValue*) item_data(n);
	if (!dev) return;
	fl_font(0, 14);
	item_size(n, (int)fl_width(dev->name), fl_height());
}

int DeviceTree::item_nb_children(int n)
{
	DeviceValue *dev = (DeviceValue*) item_data(n);
	if (!dev) return 0;
	return dev->nb_child;
}

int DeviceTree::item_has_children(int n)
{
	DeviceValue *dev = (DeviceValue*) item_data(n);
	if (!dev) return 0;
	if (!dev->nb_child) return -1;
	return 1;
}

void *DeviceTree::item_get_child(int n, int c)
{
	DeviceValue *dev = (DeviceValue*) item_data(n);
	if (!dev) return NULL;
	if (c >= dev->nb_child) return NULL;
	c--;
	return dev->child[c];
}

void DeviceTree::item_free(int n)
{
	DeviceValue *dev = (DeviceValue*) item_data(n);
	if (!dev) return;
}

void DeviceTree::item_event(void)
{
	int n = get_selection();
	DeviceValue *dev = (DeviceValue*) item_data(n);
}


//
// End of "$Id: $".
//

