//
// "$Id: $"
//
//
//                   Copyright (c) 2003, O'ksi'D
//                      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of O'ksi'D nor the names of its contributors
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//

#include "Device.h"
#include "Gui.h"
#include "sexpr.h"
#include <FL/Fl_Window.H>
#include <FL/Fl.H>
#include <FL/fl_ask.H>
#include <FL/fl_draw.H>
#include <FL/Fl_Preferences.H>
#include <stdlib.h>
#include <stdio.h>
#ifdef WIN32
#define snprintf _snprintf
#define S_IRGRP 0
#define S_IROTH 0
#include <io.h>
#else
#include <unistd.h>
#endif
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <FL/Fl_File_Chooser.H>

extern Gui *gui;

static void map_sel_but_cb(Fl_Widget *w, void *d)
{
	((Devices*) d)->open_map();
}
 
static void map_save_but_cb(Fl_Widget *w, void *d)
{
	((Devices*) d)->save_map();
}

static void map_new_but_cb(Fl_Widget *w, void *d)
{
	((Devices*) d)->new_map();
}

Devices::Devices(int X, int Y, int W, int H, const char *T) :
	Fl_Group(X, Y, W, H, T)
{
	char buf[2048];

	map_file_inp = new Fl_Input(X + 5, Y + 4, 400, 25);
	map_sel_but = new Fl_Button(X + 410, Y + 4, 40, 25, "open");
	map_sel_but->callback(map_sel_but_cb, this);
	map_save_but = new Fl_Button(X + 455, Y + 4, 40, 25, "save");
	map_save_but->callback(map_save_but_cb, this);
	map_new_but = new Fl_Button(X + 500, Y + 4, 40, 25, "new");
	map_new_but->callback(map_new_but_cb, this);

	res = new Fl_Box( X + W - 7, Y + H - 7, 2, 2);
	res->hide();
	
	nb_fix = 0;
	fix = NULL;
	mal_fix = 0;
	
	clear_map();

	table = new DevicesTable(X + 0, Y + 30, W - 0, H - 30, this);

	
	hide();
	end();
	resizable(res);
	changed = 0;

	snprintf(buf, 2047, "%s/default.map", gui->work_dir);
	map_file_inp->value(buf);	
	load_map();
}

Devices::~Devices()
{
	while (nb_fix > 0) {
		delete (fix[--nb_fix]);
	}
	free(fix);
}

void Devices::load_word(const char *word, int len, int depth, int closed)
{
	char *b = NULL;

	switch (stage1) {
	case -1:
		return;
	case 0:
		if (!strncmp(word, "DmxStep_Map_File", len) && 
			len == (int)strlen("DmxStep_Map_File")) 
		{
			stage1 = 2;
		} else {
			stage1 = -1;
		}
		return;
	case 2:
		if (!strncmp(word, "nb_fix", len) && len == (int)strlen("nb_fix")) {
			stage1 = 3;
		}
		return;
	case 3:
		if (closed == 0) {
			stage1 = 4;
			stage2 = 0;
			b = (char*) malloc(len + 1);
			strncpy(b, word, len);
			b[len] = 0;
			stage_max_fix = atol(b);
			free(b);
			stage_fix = -1;
		} else {
			stage1 = -1;
		}
		return;
	case 4:
		if (depth == 2) {
			
			if (closed > 0) {
				stage2 = 0;
				stage_fix++;	
			}
			if (stage_fix >= stage_max_fix) {
				stage1 = 5;
				stage2 = 0;
				return;
			}
			switch (stage2) {
			case 0:
				b = (char *)malloc(len + 1);
				strncpy(b, word, len);
				b[len] = 0;
				set_value(stage_fix, 1, b);
				free(b);
				break;
			case 1:
				b = (char *)malloc(len + 1);
				strncpy(b, word, len);
				b[len] = 0;
				set_value(stage_fix, 0, b);
				free(b);
				break;		
			case 2:
				b = (char *)malloc(len + 1);
				strncpy(b, word, len);
				b[len] = 0;
				fix[stage_fix]->set_universe(atol(b));
				free(b);
				break;		
			case 3:
				b = (char *)malloc(len + 1);
				strncpy(b, word, len);
				b[len] = 0;
				fix[stage_fix]->set_base(atol(b));
				free(b);
				break;		
			}
			stage2++;
		}
	}
}

void Devices::load_map()
{
	char *buf, *buf1;
	char *ptr1, *ptr2;
	int mlen = 2048;
	int depth = 0;
	int closed = 0;
	FILE *fd;
	int rl;
	
	Gui::lock();

	clear_map();
	if (!map_file_inp->value() || !map_file_inp->value()[0]) {
		char buf[2048];
		snprintf(buf, 2048, "%s/Maps/default.map", gui->work_dir);
		map_file_inp->value(buf);
	}
	fl_make_path_for_file(map_file_inp->value());

	fd = fl_fopen(map_file_inp->value(), "rb");
	if (!fd) {
		//fl_alert("Unable to open DMX map !");
		Gui::unlock();
		return;
	}

	stage1 = 0;
	stage2 = 0;

	buf = (char*) malloc(mlen);
	buf1 = (char*) malloc(mlen);

	ptr1 = buf;
	while (!feof(fd)) {
		char *end;
		
		if (mlen < ptr1 - buf + 512) {
			mlen += 1024;
			buf = (char*)realloc(buf, mlen);
			buf1 = (char*)realloc(buf1, mlen);
		}
		ptr1 = buf;
		rl = fread(ptr1, 1, mlen - (ptr1 - buf), fd);
		ptr2 = ptr1 = buf;
		end = ptr1 + rl;
		
		while (ptr1 < end) {
			while (ptr1 < end && isspace(ptr1[0])) ptr1++;
			if (ptr1 < end) ptr2 = ptr1;
			if (ptr1 < end && ptr1[0] == '(') { 
				depth++;
				ptr1++;
				ptr2 = ptr1;
			} else if (ptr1 < end && ptr1[0] == ')') {
				depth--;
				ptr1++;
				ptr2 = ptr1;
				closed++;
			} else if (ptr1 < end && ptr1[0] == '"') {
				ptr1++;
				while (ptr1 < end && ptr1[0] != '"') ptr1++;
				if (ptr1 < end) {
					int l;
					ptr1++;
					l = sexpr_unquote(ptr2, ptr1 - ptr2,
							buf1, mlen);
					load_word(buf1, l, depth, closed);
					closed = 0;
					ptr2 = ptr1;
				}		
			} else if (ptr1 < end) {
				while (ptr1 < end && 
					!isspace(ptr1[0]) && ptr1[0] != '(' &&
					ptr1[0] != ')') ptr1++;
				if (ptr1 < end) {
					load_word(ptr2, ptr1 - ptr2, depth, closed);
					closed = 0;
					ptr2 = ptr1;
				}		
			}
		}

		memmove(buf, ptr2, rl - (ptr2 - buf));
	}

	free(buf);
	free(buf1);
	fclose(fd);
	changed = 0;
	Gui::unlock();
}

void Devices::open_map()
{
	if (changed) save_map();
	changed = 0;
	Fl_File_Chooser fc(map_file_inp->value(), 
		"Devices map (*.map)", Fl_File_Chooser::SINGLE, "Open Devices map");
	fc.show();
	while (fc.visible())
		Fl::wait();
	
	if (fc.count() > 0 && fc.value(0) ) {
		map_file_inp->value(fc.value(0));
		load_map();
	} 
	redraw();
}

void Devices::save_map()
{
	FILE *fd;
	int i;

	if (!map_file_inp->value() || !map_file_inp->value()[0]) {
		Fl_File_Chooser fc(map_file_inp->value(), 
			"Devices map (*.map)", Fl_File_Chooser::CREATE, "Save as map ...");
		fc.show();
		while (fc.visible())
			Fl::wait();
	
		if (fc.count() > 0 && fc.value(0) ) {
			map_file_inp->value(fc.value(0));
		} else {
			return;
		} 
	}

	fd = fl_fopen(map_file_inp->value(), "wb");
	if (!fd) {
		fl_alert("Unable to save DMX map !");
		return;
	}

	fprintf(fd, "(DmxStep_Map_File\n");
	fprintf(fd, "  (version  1.0.0)\n");
	fprintf(fd, "  (nb_fix  %d)\n", nb_fix);
	for (i = 0; i < nb_fix; i++) {
		fix[i]->dump_settings(fd);
	}

	fprintf(fd, ")\n");

	fclose(fd);

	changed = 0;
}

void Devices::new_map()
{
	if (changed) save_map();
	changed = 0;
	map_file_inp->value("");
	clear_map();
	redraw();
}

void Devices::clear_map()
{
	int i;

	Gui::lock();

	while (nb_fix > 0) {
		delete (fix[--nb_fix]);
	}
	if (nb_fix < 32) {
		fix = (Fixture **) malloc(sizeof(Fixture*) * 32);
		mal_fix = 32;
	}
	for (i = 0; i < mal_fix; i++) {
		fix[i] = NULL;
	}
	for (i = 0; i < 32; i++) {
		fix[i] = new Fixture();
	}
	nb_fix = 32;
	
	Gui::unlock();
}

void Devices::grow()
{
	int i;
	Gui::lock();

	nb_fix += 8;
	if (nb_fix > mal_fix) {
		mal_fix = nb_fix;
		fix = (Fixture **) realloc(fix, sizeof(Fixture*) * mal_fix);	
	}

	for (i = nb_fix - 8; i < nb_fix; i++) {
		fix[i] = new Fixture();
	}
	table->rows(nb_fix);
	Gui::unlock();
}

void Devices::set_value(int r, int c, const char *v)
{
	Fixture *f;
	int i;
	char *b = NULL;

	Gui::lock();

	gui->suspend();

	changed = 1;

	while (r >= nb_fix) grow();
	
	switch (c) {
	case 0:
		
		for (i = 0; i < nb_fix && v && v[0]; i++) {
			if (i != r && fix[i]->name &&
				!strcmp(b ? b : v, fix[i]->name)) 
			{
				free(b);
				b = (char*)malloc(strlen(fix[i]->name) + 2);
				sprintf(b, "%s*", fix[i]->name);	
			}
		}
		fix[r]->set_name(b ? b : v);
		free(b);
		break;
	case 1:	
		f = fix[r];
		fix[r] = new Fixture(v);
		if (f) {
			fix[r]->set_base(f->base_address);
			fix[r]->set_name(f->name);
			fix[r]->set_universe(f->universe);
			delete(f);
		}
		break;
	case 2:
		fix[r]->set_universe(atol(v + 4) - 1);
		break;
	case 3:
		fix[r]->set_base(atol(v) - 1);
		break;
	} 	 

	gui->resume();

	Gui::unlock();
}

const char *Devices::get_value(int r, int c)
{
	static char buf[256];

	if (r >= nb_fix) return "";
	
	switch (c) {
	case 0:
		if (!fix[r]->name) return "";
		return fix[r]->name;
	case 1:
		if (!fix[r]->type) return "<none>";
		return fix[r]->type->type();
	case 2:
		snprintf(buf, 255, "Uout%02d", fix[r]->universe + 1);
		return buf;
	case 3:
		snprintf(buf, 255, "%03d", fix[r]->base_address + 1);
		return buf;
	} 
	return "";
}

//
// End of "$Id: $".
//

