//
// "$Id: $"
//
//
//                   Copyright (c) 2003, O'ksi'D
//                      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of O'ksi'D nor the names of its contributors
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//

#include "Device.h"
#include "FixtureType.h"
#include "Gui.h"
#include "sexpr.h"
#include <FL/Fl_Window.H>
#include <FL/Fl.H>
#include <FL/fl_ask.H>
#include <FL/fl_draw.H>
#include <FL/Fl_Preferences.H>
#include <stdlib.h>
#include <stdio.h>
#ifdef WIN32
#define snprintf _snprintf
#define S_IRGRP 0
#define S_IROTH 0
#include <io.h>
#else
#include <unistd.h>
#endif
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <FL/Fl_File_Chooser.H>


static Fl_Menu_Item universe_item[] = {
	{"Uout01"},
	{"Uout02"},
	{"Uout03"},	
	{"Uout04"},
	{"Uout05"},
	{"Uout06"},	
	{"Uout07"},
	{"Uout08"},
	{"Uout09"},	
	{"Uout10"},
	{"Uout11"},
	{"Uout12"},	
	{"Uout13"},
	{"Uout14"},
	{"Uout15"},	
	{"Uout16"},
	{"Uout17"},
	{"Uout18"},	
	{"Uout19"},
	{"Uout20"},
	{"Uout21"},	
	{"Uout22"},	
	{"Uout23"},
	{"Uout24"},
	{"Uout25"},	
	{"Uout26"},
	{"Uout27"},
	{"Uout28"},	
	{"Uout29"},
	{"Uout30"},
	{"Uout31"},	
	{"Uout32"},	
	{0}
};

static void enter_cb(Fl_Widget*, void* v)
{ 
	((DevicesTable*)v)->set_value(); 
}

DevicesTable::DevicesTable(int x, int y, int w, int h, Devices *d) : 
	Fl_Table(x, y, w, h)
{
	dev = d;
	


	input = new Fl_Input(w/2,h/2,0,0);
	input->hide();
	input->callback(enter_cb, (void*)this);
	input->when(FL_WHEN_ENTER_KEY_ALWAYS);

	universe = new Fl_Choice(w/2, h/2, 0, 0);
	universe->hide();
	universe->callback(enter_cb, (void*)this);
	universe->menu(universe_item);

	channel = new Fl_Int_Input(w/2, h/2, 0, 0);
	channel->hide();
	channel->callback(enter_cb, (void*)this);
	channel->when(FL_WHEN_ENTER_KEY_ALWAYS);

	type = new Fl_Menu_Button(w/2, h/2, 0, 0);
	type->hide();
	type->callback(enter_cb, (void*)this);
	type->menu(FixtureType::get_menu());
	if (type->menu()) type->label(type->menu()[0].text);


	callback(&event_callback, (void*)this);
	when(FL_WHEN_NOT_CHANGED|when());

	(new Fl_Box(9999,9999,0,0))->hide();  // HACK: prevent flickering in Fl_Scroll

	cols(4);
	rows(32);
	col_header(1);
	col_resize(1);
	row_resize(1);
	row_height_all(25);

	col_header_height(30);
	col_width(0, 300);
	col_width(1, 120);

	end();
}

void DevicesTable::set_value()
{
	const char *v;

	if (type->visible()) {
		type->label(type->menu()[type->value()].text);
		v = type->menu()[type->value()].text;
	}
	if (input->visible()) v = input->value();
	if (channel->visible()) v = channel->value();
	if (universe->visible()) v = universe->menu()[universe->value()].text;	
	
	dev->set_value(row_edit, col_edit, v);
}

// Callback whenever someone clicks on different parts of the table
void DevicesTable::event_callback(Fl_Widget*, void *data)
{
    DevicesTable *o = (DevicesTable*)data;
    o->event_callback2();
}

void DevicesTable::event_callback2()
{
    int R = callback_row(),
        C = callback_col();
    TableContext context = callback_context();



    switch ( context )
    {
	case CONTEXT_CELL:
	{
            switch (Fl::event()) 
	    { 
		case FL_PUSH:
		    Fl::event_clicks(0);
		    //FALLTHROUGH

		case FL_KEYBOARD:
		    if (Fl::event() == FL_KEYBOARD && Fl::e_length == 0)
		        { return; }
		    do_cb();
		    hide_inp();

		    if (R > rows() - 2) rows(rows() + 8);

		    row_edit = R;
		    col_edit = C;
		    set_selection(R, C, R, C);
		    int XX,YY,WW,HH;
		    find_cell(CONTEXT_CELL, R, C, XX, YY, WW, HH);
		    if (C == 0) {
			input->resize(XX,YY,WW,HH);
			input->value(dev->get_value(R, C));
			input->show();
			input->take_focus();
		
			if (Fl::event() == FL_KEYBOARD && Fl::e_text[0] != '\r') { 
				input->handle(Fl::event()); 
			}
		    } else if (C == 1) {
			const char *v;
			int d, m;
			v = dev->get_value(R, C);
			type->resize(XX,YY,WW,HH);
			type->show();
			type->take_focus();
			m = 0;
			d = 1;
			type->value(0);
			while (d > 0) {
				if (!type->menu()[m].text) {
					d--;
				} else if (type->menu()[m].flags & FL_SUBMENU) {
					d++;
				} else if (!strcmp(type->menu()[m].text, v)) {
					type->value(m);
					break;
				}
				m++;
			}
			type->label(type->menu()[type->value()].text);
			if (Fl::event() == FL_KEYBOARD && Fl::e_text[0] != '\r') { 
				type->handle(Fl::event()); 
			}
		    } else if (C == 2) {
			const char *v;
			int d, m;
			v = dev->get_value(R, C);
			universe->resize(XX,YY,WW,HH);
			m = 0;
			d = 1;
			universe->value(0);
			while (d > 0) {
				if (!universe->menu()[m].text) {
					d--;
				} else if (!strcmp(universe->menu()[m].text, v)) {
					universe->value(m);
					break;
				}
				m++;
			}
			universe->show();
			universe->take_focus();
			if (Fl::event() == FL_KEYBOARD && Fl::e_text[0] != '\r') { 
				universe->handle(Fl::event()); 
			}
		    } else if (C == 3) {
			channel->resize(XX,YY,WW,HH);
			channel->value(dev->get_value(R, C));
			channel->show();
			channel->take_focus();
			if (Fl::event() == FL_KEYBOARD && Fl::e_text[0] != '\r') { 
				channel->handle(Fl::event()); 
			}
		    }
		    return;
	    }
	    return;
	}

	case CONTEXT_ROW_HEADER:
	case CONTEXT_COL_HEADER:
	    do_cb();
	    hide_inp();
	    return;

	case CONTEXT_TABLE:
	    if (R < 0 && C < 0)
	    {
		do_cb();
		hide_inp();
	    }
	    return;
	
	default:
	    return;
    }
}

void DevicesTable::do_cb()
{
	if (type->visible()) type->do_callback();
	if (input->visible()) input->do_callback();
	if (channel->visible()) channel->do_callback();
	if (universe->visible()) universe->do_callback();
}

void DevicesTable::hide_inp()
{
	type->hide();
	input->hide();
	channel->hide();
	universe->hide();
}

// Handle drawing all cells in table
void DevicesTable::draw_cell(TableContext context, 
			    int R, int C, int X, int Y, int W, int H)
{
    static char s[30];

    if (input->visible() || universe->visible() || 
		type->visible() || channel->visible())
    {
        if (Fl::focus() != input && Fl::focus() != universe &&
	    Fl::focus() != type && Fl::focus() != channel) 
        {
	    do_cb();
	    hide_inp();
        }
    }

    switch ( context )
    {
	case CONTEXT_STARTPAGE:
	    // Get kb nav + mouse 'selection region' for use below
	    get_selection(s_top, s_left, s_bottom, s_right);	
	    break;

	case CONTEXT_COL_HEADER:
	    fl_font(FL_HELVETICA | FL_BOLD, 14);
	    fl_push_clip(X, Y, W, H);
	    {
		fl_draw_box(FL_THIN_UP_BOX, X, Y, W, H, col_header_color());
		fl_color(FL_BLACK);
		switch (C) {
		case 0:
			fl_draw("Device name", X, Y, W, H, FL_ALIGN_CENTER);
			break;
		case 1:
			fl_draw("Type", X, Y, W, H, FL_ALIGN_CENTER);
			break;
		case 2:
			fl_draw("Universe", X, Y, W, H, FL_ALIGN_CENTER);
			break;
		case 3:
			fl_draw("Base \naddress", X, Y, W, H, FL_ALIGN_CENTER);
			break;
			
		}
	    }
	    fl_pop_clip();
	    return;

	case CONTEXT_ROW_HEADER:
	    fl_font(FL_HELVETICA | FL_BOLD, 14);
	    fl_push_clip(X, Y, W, H);
	    {
/*
		fl_draw_box(FL_THIN_UP_BOX, X, Y, W, H, row_header_color());
		fl_color(FL_BLACK);
		fl_draw(dev->get_value(R, C), X, Y, W, H, FL_ALIGN_LEFT);
*/
	    }
	    fl_pop_clip();
	    return;

	case CONTEXT_CELL:
	{

	    if (R == row_edit && C == col_edit && 
		(input->visible() || universe->visible() || 
		type->visible() || channel->visible()))
	    { 
		return; 
	    }

	    // BACKGROUND
	    fl_push_clip(X, Y, W, H);

	    // Keyboard nav and mouse selection highlighting
	    if (R >= s_top && R <= s_bottom && C >= s_left && C <= s_right)
                { fl_draw_box(FL_THIN_UP_BOX, X, Y, W, H, FL_YELLOW); }
            else
	        { fl_draw_box(FL_THIN_UP_BOX, X, Y, W, H, FL_WHITE); }

	    fl_pop_clip();
	    // TEXT
	    fl_push_clip(X+3, Y+3, W-6, H-6);
	    {
		fl_font(FL_HELVETICA, 14);
		fl_color(FL_BLACK);
		fl_draw(dev->get_value(R, C), X + 3, Y + 3, W - 6, H - 6, FL_ALIGN_LEFT);

	    }
	    fl_pop_clip();
	    return;
	}

	case CONTEXT_RC_RESIZE:
	{
	    if (!input->visible() && !type->visible() && 
		!channel->visible() && !universe->visible()) return;
	    find_cell(CONTEXT_TABLE, row_edit, col_edit, X, Y, W, H);
	    input->resize(X,Y,W,H);
	    type->resize(X,Y,W,H);
	    channel->resize(X,Y,W,H);
	    universe->resize(X,Y,W,H);
	    return;
	}
	default:
	    return;
    }
}


//
// End of "$Id: $".
//

