//
// "$Id: $"
//
//
//                   Copyright (c) 2004, O'ksi'D
//                      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of O'ksi'D nor the names of its contributors
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//


#include <FL/Fl.H>
#include "Fader.h"
#include "FixtureType.h"
#include <FL/fl_draw.H>
#ifdef WIN32
#define snprintf _snprintf
#define S_IRGRP 0
#define S_IROTH 0
#include <io.h>
#else
#include <unistd.h>
#endif

Fader::Fader(int X, int Y, int W, int H) : Fl_Slider(X,Y,W,H) {
	maximum(0);
	minimum(255);
	type(FL_VERT_NICE_SLIDER);
	selection_color(FL_YELLOW);
	step(1.0);	
};
	
void Fader::draw(void)
{
	char b[4];
	int Y, X;
	
	Fl_Slider::draw();

	snprintf(b, 4, "%03d", (int)value());
	fl_color(FL_WHITE);
	fl_font(FL_HELVETICA_BOLD, 10);
	
	if (b[0] != '0') {
		Y = y()  + h() - (1 + 2 * fl_height() + fl_descent());
	} else {
		Y = y() + 2 + fl_height() - fl_descent();
	}
	X = x() + 8;

	fl_rectf(X - 4, Y - 9, 8, 32);
	fl_color(FL_BLACK);
	fl_draw(b, 1, X - (int)fl_width(b, 1) / 2  , Y);
	Y += fl_height() - 2;
	fl_draw(b + 1, 1, X - (int)fl_width(b + 1, 1) / 2 , Y);
	Y += fl_height() - 2;
	fl_draw(b + 2, 1, X - (int)fl_width(b + 2, 1) / 2 , Y);
	
}


static void channel_group_cb(Fl_Widget *w, void *data)
{
	ChannelGroup *d = (ChannelGroup*)data;
	((FixtureTypeChannelGroup*)d->parent())->do_it(d);	
}

static void dimmer_dimmer_cb(Fl_Widget *w, void *data)
{
	Dimmer *d = (Dimmer*)data;

	// TODO : set preset !


	channel_group_cb(w, data);	
}

static void dimmer_pre_cb(Fl_Widget *w, void *data)
{
	Dimmer *d = (Dimmer*)data;

	// TODO : set preset !


	channel_group_cb(w, data);	
}

static void dimmer_mix_cb(Fl_Widget *w, void *data)
{
	Dimmer *d = (Dimmer*)data;
	switch (d->mix_mode->label()[0]) {
	default:
	case 'L':
		d->mix_mode->label("HTP");
		break;
/*	case 'H':
		d->mix_mode->label("ADD");
		break;
	case 'A':
		d->mix_mode->label("SUB");
		break;
	case 'S':
*/
	case 'H':
		d->mix_mode->label("LTP");
		break;
	}
	channel_group_cb(w, data);	
}

Dimmer::Dimmer(int X, int Y, int W, int H) : ChannelGroup(X,Y,W,H)
{
	resize(x(), y(), 16, 16 + 256 + 15 + 10 + 16);
	pre = new Fl_Button(x(), y(), 16, 16);	
	pre->callback(dimmer_pre_cb, this);
	dimmer = new Fader(x(), y() + 16, 16, 256 + 15);
	dimmer->callback(dimmer_dimmer_cb, this);
	mix_mode = new Fl_Button(x(), y() + 16 + 256 + 15, 16, 10);
	mix_mode->labelsize(8);
	mix_mode->label("LTP");
	mix_mode->labelcolor(FL_WHITE);
	mix_mode->box(FL_FLAT_BOX);
	mix_mode->color(FL_DARK_BLUE);
	mix_mode->callback(dimmer_mix_cb, this);
	fade_curve = new Fl_Button(x(), y() + 16 + 256 + 15 + 10, 16, 16);
	fade_curve->callback(channel_group_cb, this);
	end();
}

void Dimmer::get(int *curve, int *mix, int *value)
{
	switch (mix_mode->label()[0]) {
	default:
	case 'L':
		*mix = 0;
		break;
	case 'H':
		*mix = 1;
		break;
	case 'A':
		*mix = 2;
		break;
	case 'S':
		*mix = 3;
		break;
	}

	*value = (int) dimmer->value();

	// TODO : fade in curve
	*curve = 0;
}

void Dimmer::set(int curve, int mix, int value)
{
	switch (curve) {
	case 0:
	default:
		break;
	};

	switch (mix) {
	case 0:
	default:
		mix_mode->label("LTP");
		break;
	case 1:
		mix_mode->label("HTP");
		break;
	case 2:
		mix_mode->label("ADD");
		break;
	case 3:
		mix_mode->label("SUB");
		break;

	};
	// TODO : preset

	if (Fl::pushed() == dimmer && Fl::event_buttons()) {
		dimmer->do_callback();
	} else { 
		dimmer->value((double) value);
	}

}

void Dimmer::preset(Fl_Menu_Item *mi)
{
}

//
// End of "$Id: $".
//

