//
// "$Id: $"
//
//
//                   Copyright (c) 2003, O'ksi'D
//                      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of O'ksi'D nor the names of its contributors
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//

#include "Fixture.h"
#include "Gui.h"
#include "sexpr.h"

#include <stdlib.h>
#include <stdio.h>
#ifdef WIN32
#define snprintf _snprintf
#define S_IRGRP 0
#define S_IROTH 0
#include <io.h>
#else
#include <unistd.h>
#endif
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

extern Gui *gui;

Fixture::Fixture()
{
	name = NULL;
	type = FixtureType::get_fixture_type(NULL);
	universe = 0;
	base_address = 0;
}

Fixture::Fixture(const char *t)
{
	name = NULL;
	type = FixtureType::get_fixture_type(t);
	universe = 0;
	base_address = 0;
}

Fixture::~Fixture()
{
	free(name);
}

void Fixture::set_name(const char *n)
{
	free(name);
	name = strdup(n);
}

void Fixture::set_universe(int u)
{
	if (u < 0 || u > 31) u = 0;
	universe = u;
}

void Fixture::set_base(int b)
{
	if (b < 0 || b > 511) b = 0;
	base_address = b;	
}

int Fixture::has_changed_virtual(int channel)
{
	return gui->out_universes[universe]->
			has_changed(base_address + channel);
}

int Fixture::get_virtual(int channel)
{
	return gui->out_universes[universe]->
			get_nf_channel(base_address + channel);
//	return type->get_virtual(universe, base_address, channel);
}

void Fixture::set_virtual(int channel, int val)
{
	gui->out_universes[universe]->
			set_channel(base_address + channel, val);

//	type->set_virtual(universe, base_address, channel, val);
}

void Fixture::dump_settings(FILE *fd)
{
	char buf[2048];

	fprintf(fd, "  (\n");

	if (type) {
		sexpr_quote(type->type(), strlen(type->type()), buf, 2048);
		fprintf(fd, "    %s\n", buf);
	} else {
		fprintf(fd, "    \"\"\n");
	}

	if (name) {
		sexpr_quote(name, strlen(name), buf, 2048);
		fprintf(fd, "    %s\n", buf);
	} else {
		fprintf(fd, "    \"\"\n");
	}

	fprintf(fd, "    %d\n", universe);

	fprintf(fd, "    %d\n", base_address);
	
	fprintf(fd, "  )\n\n");
}

FixtureTypeChannelGroup *Fixture::get_panel(FixtureTypeCb *cb, void *data)
{
	return type->get_panel(cb, data);
}

//
// End of "$Id: $".
//

