//
// "$Id: $"
//
//
//                   Copyright (c) 2003, O'ksi'D
//                      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of O'ksi'D nor the names of its contributors
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//

#include "FixtureTree.h"
#include "Device.h"
#include <FL/fl_draw.H>
#include "Gui.h"
#include <stdlib.h>
#include <stdio.h>
#ifdef WIN32
#include <io.h>
#else
#include <unistd.h>
#endif
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
extern Gui *gui;

static void item_cb(Fl_Widget *w, void *d)
{
	((FixtureTree*)d)->item_event();
}

FixtureTree::FixtureTree(int X, int Y, int W, int H, const char* T) : 
	Fl_Tree (X, Y, W, H, T)
{
	callback(item_cb, this);
	
	cb = NULL;
}

void FixtureTree::item_draw(int n, int X, int Y, int CX, int CY, int CW, int CH)
{
	int nbc, nbs;
	int l;
	const char *name;
	int f = (int) item_data(n);
	int off = 0;

	nbc = nbs = 0;
	if (f > gui->devices->nb_fix) return;
	if (f >= 0) {
		name = gui->devices->fix[f]->name;
		if (!name) return;
	} else {
		name = "Fixtures";
		nbc = 8;
	}
	l = levels[n] - 1;

	
	while (nbs <= l && name[nbc]) {
		if (name[nbc] == '/') nbs++;
		if (nbs < l) off = nbc + 2;
		if (nbs <= l) nbc++; 
	}

	fl_font(0, 14);
 
	if (item_flags(n) & FLAG_SELECTED && !(item_flags(n) & FLAG_FOLDER)) {
		fl_color(FL_YELLOW);
		fl_rectf(X, Y, (int)fl_width(name + off, nbc - off), fl_height());
	}
	if (Fl::focus() == this && get_selection() == n) {
		fl_color(FL_GREEN);
		dot_rect(X, Y, (int)fl_width(name + off, nbc - off), fl_height());
	}
	fl_color(FL_BLACK);
	Y += fl_height() - fl_descent();
	fl_draw(name + off, nbc - off, X, Y);
}

void FixtureTree::item_measure(int n)
{
	int nbc, nbs;
	int l;
	int off = 0;
	const char *name;
	int f = (int) item_data(n);

	nbc = nbs = 0;

	if (f > gui->devices->nb_fix) return;
	if (f >= 0) {
		name = gui->devices->fix[f]->name;
		if (!name) return;
	} else {
		name = "Fixtures";
		nbc = 8;
	}
	l = levels[n] -1;

	
	while (nbs <= l && name[nbc]) {
		if (name[nbc] == '/') nbs++;
		if (nbs < l) off = nbc + 2;
		if (nbs <= l) nbc++;
		
	}

	fl_font(0, 14);
	item_size(n, (int)fl_width(name + off, nbc - off), fl_height());
}

int FixtureTree::item_nb_children(int n)
{
	int nbc, nbs;
	int l;
	const char *name;
	int f = (int) item_data(n);
	int i;
	int nb;


	if (f < 0) {
		nb = 0;
		for (i = 0; i < gui->devices->nb_fix; i++) {
			const char *name = gui->devices->fix[i]->name;
			nbc = 0;
			if (!name || !name[0]) continue;
			while (name[nbc] && name[nbc] != '/') nbc++;
			if (!name[nbc]) {
				nb++;
			} else {
				int j;
				nbc++;
				nb++;
				for (j = i + 1; j < gui->devices->nb_fix; j++) {
					if (!strncmp(name, gui->devices->fix[j]->name, nbc)) {
						nb--;
						break;
					}
				}
			}
		}
		return nb;
	}

	if (f > gui->devices->nb_fix) return 0;
	name = gui->devices->fix[f]->name;
	if (!name) return 0;

	l = levels[n] - 1;

	nbc = nbs = 0;
	while (nbs <= l && name[nbc]) {
		if (name[nbc] == '/') nbs++;
		if (nbs <= l) nbc++;
	}
	
	if (!name[nbc]) return 0;

	nbc++;

	nb = 0;
	for (i = 0; i < gui->devices->nb_fix; i++) {
		const char *na = gui->devices->fix[i]->name;
		if (!strncmp(name, na, nbc)) {
			int nc = nbc;
			while (na[nc] && na[nc] != '/') nc++;
			if (na[nc] == '/') {
				int j;
				nc++;
				for (j = i + 1; j < gui->devices->nb_fix; j++) {
					if (!strncmp(na, gui->devices->fix[j]->name, nc)) {
						nb--;
						break;
					}
				}
			}
			nb++;
		}
	}
	return nb;
}

int FixtureTree::item_has_children(int n)
{
	int nbc, nbs;
	int l;
	const char *name;
	int f = (int) item_data(n);

	if (f < 0) return 1;

	if (f > gui->devices->nb_fix) return 0;
	name = gui->devices->fix[f]->name;
	if (!name || !name[0]) return -1;

	l = levels[n] - 1;

	nbc = nbs = 0;
	while (nbs <= l && name[nbc]) {
		if (name[nbc] == '/') nbs++;
		if (nbs <= l) nbc++;
	}

	if (!name[nbc]) return -1;
	return 1;
}

void *FixtureTree::item_get_child(int n, int c)
{
	int nbc, nbs;
	int l;
	const char *name;
	int f = (int) item_data(n);
	int i;
	int nb;

	if (f > gui->devices->nb_fix) return NULL;
	if (f < 0) {
		nb = 0;
		for (i = 0; i < gui->devices->nb_fix; i++) {
			const char *name = gui->devices->fix[i]->name;
			nbc = 0;
			if (!name || !name[0]) continue;
			while (name[nbc] && name[nbc] != '/') nbc++;
			if (!name[nbc]) {
				
				nb++;
			} else {
				int j;
				nbc++;
				for (j = i + 1; j < gui->devices->nb_fix; j++) {
					if (!strncmp(name, gui->devices->fix[j]->name, nbc)) {
						nb--;
						break;
					}
				}
				nb++;
			}
			if (nb == c) return (void*)i;
		}
		return NULL;
	} else {
		name = gui->devices->fix[f]->name;
		if (!name) return NULL;
	}

	l = levels[n] - 1;

	nbc = nbs = 0;
	while (nbs <= l && name[nbc]) {
		if (name[nbc] == '/') nbs++;
		if (nbs <= l) nbc++;
	}
	
	if (!name[nbc]) return NULL;

	nbc++;

	nb = 0;
	for (i = 0; i < gui->devices->nb_fix; i++) {
		const char *na = gui->devices->fix[i]->name;
		if (!strncmp(name, na, nbc)) {
			int nc = nbc;
			while (na[nc] && na[nc] != '/') nc++;
			if (na[nc] == '/') {
				int j;
				nc++;
				for (j = i + 1; j < gui->devices->nb_fix; j++) {
					if (!strncmp(na, gui->devices->fix[j]->name, nc)) {
						nb--;
						break;
					}
				}
			}
			nb++;
			if (nb == c) return (void*)i; 
		}
	}
	return NULL;
}

void FixtureTree::item_free(int n)
{

}

void FixtureTree::item_event(void)
{
    // get the id of the element of which the user has clicked
    int n = get_selection();
    if (Fl::event_clicks()) {
      // is it already openned ?
      if (item_flags(n) & FLAG_OPEN) {
        // yes, close it
        item_close(n);
      } else {
        // no, open it
        item_open(n);
      }
      Fl::event_clicks(0);
    } else {
	
    }	
    if (cb) cb(this, cb_data);
}

void FixtureTree::set_cb(FixtureTreeCb* c, void *d)
{
	cb = c;
	cb_data = d;
}

//
// End of "$Id: $".
//

