//
// "$Id: $"
//
//
//                   Copyright (c) 2004, O'ksi'D
//                      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of O'ksi'D nor the names of its contributors
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//
#include "Step.h"
#include "FixtureType.h"
#include "Gui.h"
#include "sexpr.h"
#include <FL/Fl_File_Chooser.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Int_Input.H>
#include <FL/Fl_Output.H>
#include <FL/Fl_Int_Input.H>
#include "Fader.h"
#include <stdlib.h>
#include <stdio.h>
#ifdef WIN32
#define snprintf _snprintf
#define S_IRGRP 0
#define S_IROTH 0
#include <io.h>
#else
#include <unistd.h>
#endif
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

extern Gui *gui;

Fl_Menu_Item *FixtureType::type_menu = NULL;
FixtureType **FixtureType::fixture_array = NULL;
int FixtureType::alloc = 0;
int FixtureType::nb_a = 0;

#define NB_TYPE 21

static char *channel_type_name[] = {
	"Free",
	"Pan (X)",
	"Tilt (Y)",
	"Pan Fine (uX)",
	"Tilt Fine (uY)",
	"Dimmer",
	"Shutter",
	"Color",
	"Rotation of colors",
	"Gobo",
	"Rotating Gobo",
	"Iris",
	"Zoom",
	"Color Cyan",
	"Color Magenta",
	"Color Yellow",
	"Color Red",
	"Color Green",
	"Color Blue",
	"Frost",
	"Focus",
	NULL
};
 
static char *find_fix(char * buf, char *file)
{
	char bof[2048];
	int nbd, i;
	struct dirent **dir = NULL;
	char *ret = NULL;


	nbd = fl_filename_list(buf, &dir, fl_alphasort);
	
	for (i = 0; i < nbd; i++) {
		if (dir[i]->d_name[0] == '.') continue;
		snprintf(bof, 2040, "%s%s", buf, dir[i]->d_name);	
		if (fl_filename_isdir(bof)) {
			ret = find_fix(bof, file);
		} else if (!strcmp(file, dir[i]->d_name)) {
			ret = strdup(bof);
		}
		if (ret) return ret;
	}
	return NULL;
}

FixtureType::FixtureType(const char *t)
{
	char buf[2048], buf1[1024];
	char *file;

	fix_type = strdup(t);

	nb_channel = 0;
	channels = NULL;

	snprintf(buf, 2040, "%s/", gui->fixture_dir);
	snprintf(buf1, 1000, "%s.fix", t);

	file = find_fix(buf, buf1);
	load_file(file);
	free(file);

	if (nb_channel < 1) {
		channels = (FixtureTypeChannel**) malloc(sizeof(FixtureTypeChannel*));
		channels[0] = new FixtureTypeChannel();
		channels[0]->desc = strdup("Standard");
		channels[0]->icon = strdup("icon/channel_type/dimmer.png");
		nb_channel = 1;
	}
	if (!icon) icon = strdup("icon/fixture/par64.png");
}

FixtureType::~FixtureType()
{
	free(fix_type);
	free(icon);
	while (nb_channel > 0) {
		nb_channel--;
		delete (channels[nb_channel]);
	}
	free(channels);
}

void FixtureType::load_file(const char *path)
{

}

FixtureType *FixtureType::get_fixture_type(const char *t)
{
	int i;

	if (!t) t = "<none>";
	for (i = 0; i < nb_a; i++) {
		if (!strcmp(t, fixture_array[i]->fix_type)) {
			return fixture_array[i];
		}
	}	

	if (nb_a >= alloc) {
		alloc += 32;
		fixture_array = (FixtureType**) realloc(fixture_array, sizeof(FixtureType*) * alloc);
	}
	fixture_array[nb_a++] = new FixtureType(t);
	return fixture_array[nb_a - 1];
}

FixtureTypeChannelGroup *FixtureType::get_panel(FixtureTypeCb *cb, void *data)
{
	FixtureTypeChannelGroup *grp;
	int i;
	int lw = 0, lh = 0;
	grp = new FixtureTypeChannelGroup(0, 0, 0, 0);
	grp->cb = cb;
	grp->data = data;
	grp->current_ = 0;

	for (i = 0; i < nb_channel; i++) {
		ChannelGroup *g;
		g = channels[i]->create_panel(grp);
		lw = lw + g->w();
		lh = g->h() > lh ? g->h() : lh;
	}
	grp->resizable(NULL);
	grp->resize(0, 0, lw, lh); 

	grp->end();

	return grp;
}

static scan_dir_for_menu(char *buf, int root_len, int *alloc, Fl_Menu_Item **type_menu)
{
	char bof[2048];
	int nbd, i;
	struct dirent **dir = NULL;

	nbd = fl_filename_list(buf, &dir, fl_alphasort);
	
	for (i = 0; i < nbd; i++) {
		if (dir[i]->d_name[0] == '.') continue;
		snprintf(bof, 2040, "%s%s", buf, dir[i]->d_name);
		
		if (fl_filename_isdir(bof)) {
			scan_dir_for_menu(bof, root_len, alloc, type_menu);
		} else if (!strcmp(".fix", bof + strlen(bof) - 4)) {
			bof[strlen(bof) - 4] = 0;
			if ((*type_menu)->add(bof + root_len, 0, NULL) > *alloc - 16) {
				*alloc *= 2;
				*type_menu = (Fl_Menu_Item*) realloc(*type_menu, sizeof(Fl_Menu_Item) * (*alloc));
			}
		}
	}
}

Fl_Menu_Item *FixtureType::get_menu(void)
{
	
	if (!type_menu) {
		int alloc = 64;
		char buf[2048];
		int root_len;
 
		type_menu = (Fl_Menu_Item*) malloc(sizeof(Fl_Menu_Item) * alloc);
		type_menu[0].text = NULL;
		fl_make_path(gui->fixture_dir);
		type_menu->add("<none>", 0, NULL);

		snprintf(buf, 2040, "%s/", gui->fixture_dir);
		root_len = strlen(buf);
		scan_dir_for_menu(buf, root_len, &alloc, &type_menu);		
	}
	
	return type_menu;
}

const char *FixtureType::type()
{
	return fix_type;
}


FixtureTypeChannel::FixtureTypeChannel()
{
	type = 0;
	icon = NULL;
	nb_preset = 0;
	preset = NULL;
	desc = NULL;
	deflt = 0;
}

FixtureTypeChannel::~FixtureTypeChannel()
{
	free(icon);
	free(desc);
	while (nb_preset > 0) {
		delete (preset[--nb_preset]);
	}
	free(preset);
}

ChannelGroup *FixtureTypeChannel::create_panel(FixtureTypeChannelGroup *grp)
{
/*
	RGB => CMY : C = 255 - R ; M = 255 - G ; Y = 255 - B 
	LTP : Latest takes priority
	HTP : Highest takes priority
	ADD : add to current value
	SUB : substract to current value
*/
	switch (type) {
	case 1:
		//break;
	default:
		return new Dimmer(grp->x() + grp->w(), grp->y(), 0, 0);
	}	
}


FixtureTypePreset::FixtureTypePreset()
{
	icon = NULL;
	deflt = 0;
	start = 0;
	end = 0;
	desc = NULL;
}

FixtureTypePreset::~FixtureTypePreset()
{
	free(icon);
	free(desc);
}

void FixtureTypeChannelGroup::copy_from_steps(StepGroup *data) {
	int i;
	int nb_stack = data->nb_stack;
	Action *stack = data->stack;
	for (i = 0; i < nb_stack; i++) {
		if (stack[i].device == id_) {
			set(stack[i].channel,
				stack[i].curve,
				stack[i].mix_mode,
				stack[i].value);
		}
	}
}

void FixtureTypeChannelGroup::copy_to_steps(int stp, StepGroup *s) {
	int cu, mi, va;
	get(current_, &cu, &mi, &va);
	s->add_action(stp, id_, current_, cu, mi, va);
}


//
// End of "$Id: $".
//

