//
// "$Id: $"
//
//
//                   Copyright (c) 2004, O'ksi'D
//                      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of O'ksi'D nor the names of its contributors
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//

#ifndef FixtureType_H
#define FixtureType_H

#include <stdio.h>
#include "Fader.h"
#include <FL/Fl_Group.H>
#include <FL/Fl_Menu_Item.H>
#include <FL/Fl_Shared_Image.H>


class FixtureTypeChannelGroup;
class StepGroup;

typedef void (FixtureTypeCb)(FixtureTypeChannelGroup *grp, void *data);

class FixtureTypeChannelGroup : public Fl_Group
{
public:
	FixtureTypeCb *cb;
	void *data;
	int id_;
	int current_;

	FixtureTypeChannelGroup(int X, int Y, int W, int H) : Fl_Group(X,Y,W,H) {

	};
	
	int id(void) { return id_; }
	void id(int i) { id_ = i; } 
	virtual void do_it(ChannelGroup* g) {
		current_ = 0;
		while (current_ < children() && child(current_) != g) current_++;
		if (cb) cb(this, data);
	}
	virtual int current(void) {return current_;}
	virtual void get(int channel, int *curve, int *mix, int *value) {
		if (channel >= children()) return;
		((ChannelGroup*)child(channel))->get(curve, mix, value);
	}
	virtual void set(int channel, int curve, int mix, int value) {
		if (channel >= children()) return;
		((ChannelGroup*)child(channel))->set(curve, mix, value);
	}
	virtual void preset(int channel, Fl_Menu_Item *mi) {
		if (channel >= children()) return;
		((ChannelGroup*)child(channel))->preset(mi);
	}
	virtual void copy_from_steps(StepGroup *data);
	virtual void copy_to_steps(int stp, StepGroup *s);
};

class FixtureTypePreset {
public:
	char *desc;
	int start;
	int end;
	int deflt;
	char *icon;

	FixtureTypePreset();
	~FixtureTypePreset();	
};

class FixtureTypeChannel {
public:
	char *desc;
	char *icon;
	int type;
	int deflt;
	int nb_preset;
	FixtureTypePreset **preset;

	FixtureTypeChannel();
	~FixtureTypeChannel();
	
	ChannelGroup* create_panel(FixtureTypeChannelGroup *grp);
};

class FixtureType {
public:
	char *fix_type;
	char *icon;
	int nb_channel;
	FixtureTypeChannel **channels;

	static Fl_Menu_Item *type_menu;
	static FixtureType **fixture_array;
	static int alloc;
	static int nb_a;

	//FixtureTypeCb *panel_cb;
	//void *panel_data;
	
	FixtureType(const char *t);
	~FixtureType();
	
	void load_file(const char *path);

	FixtureTypeChannelGroup *get_panel(FixtureTypeCb *cb, void *data);
	static Fl_Menu_Item *get_menu(void);
	static FixtureType *get_fixture_type(const char *t);

	const char *type(void);
};



#endif // FixtureType_H

//
// End of "$Id: $".
//

