//
// "$Id: $"
//
//
//                   Copyright (c) 2003, O'ksi'D
//                      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of O'ksi'D nor the names of its contributors
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//

#ifndef Gui_H
#define Gui_H

#if defined(WIN32)
#include <windows.h>
#include <process.h>
#include <winsock.h>
#else
#include <pthread.h>
#endif


#if defined(WIN32)
typedef unsigned long Fl_Thread;

static int fl_create_thread(Fl_Thread& t, void *(*f) (void *), void* p)
{
        return t = (Fl_Thread)_beginthread(
                        (void( __cdecl * )( void * ))f, 0, p);
}

#else

typedef pthread_t Fl_Thread;

static int fl_create_thread(Fl_Thread& t, void *(*f) (void *), void* p)
{
        return pthread_create((pthread_t*)&t, 0, f, p);
}
#endif

#include <FL/Fl_Menu_Bar.H>
#include <FL/Fl_Output.H>
#include <FL/Fl_Tabs.H>
#include "Universe.h"
#include "Sequence.h"
#include "Devices.h"
#include "DevIo.h"
#include "Universes.h"
#include "CueEdit.h"
#include "ListEdit.h"
#include "ClockEdit.h"

class Gui : public Fl_Tabs {
public:
#ifdef WIN32
	static CRITICAL_SECTION cs;
#endif
	Fl_Menu_Bar *menu_bar;	// in main.cpp
	Fl_Output *status;
	int jump;

	char *work_dir;
	char *fixture_dir;
	
	Universe *in_universes[32];
	Universe *out_universes[32];
	Sequence *sequence;
	Devices *devices;
	Universes *universes;
	CueEdit *cue_edit;
	//ListEdit *list_edit;
	ClockEdit *clock_edit;

	Gui(int X, int Y, int W, int H);

	void suspend();
	void resume();
	static void lock();
	static void unlock();
};


#endif // Gui_H

//
// End of "$Id: $".
//

