//
// "$Id: $"
//
//
//                   Copyright (c) 2004, O'ksi'D
//                      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of O'ksi'D nor the names of its contributors
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//

#include "List.h"
#include "Gui.h"
#include "sexpr.h"

#include <stdlib.h>
#include <stdio.h>
#ifdef WIN32
#define snprintf _snprintf
#define S_IRGRP 0
#define S_IROTH 0
#include <io.h>
#else
#include <unistd.h>
#endif
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

extern Gui *gui;

List::List()
{
	char buf[2048];
	snprintf(buf, 2047, "%s/cueuence/", gui->work_dir);
	work_dir_len = strlen(buf);
	changed = 0;
	file_name = NULL;
	live = 1;
	time_length = 0;
	ma_cue = 0;
	nb_cue = 0;
	cue = NULL;
	bpm = 6000;
}

List::~List()
{
	free(file_name);
	//free(divisor);
	while (--ma_cue >= 0) {
		//delete(cue[ma_cue]);
	}
	free(cue);
}

int List::load(const char *file)
{
	SEXPR *fd;
	int state;
	int no_read;
	int i;
	char buf[2048];

	Gui::lock();

	snprintf(buf, 2047, "%s/cueuence/", gui->work_dir);
	work_dir_len = strlen(buf);

	if (file) {
		free(file_name);
		file_name = strdup(file);
	}

	if (!file_name) return -1;

	fd = sexpr_open(file_name);
	if (!fd) {
		return -2;
	}

	state = 0;
	no_read = 0;
	while (no_read || !sexpr_read(fd)) {
		no_read = 0;
		//printf("%d %d %d %s %d\n", sexpr_depth(fd), sexpr_closed(fd), 
		//	sexpr_element_id(fd), sexpr_text(fd), state);
		switch (state) {
		case 0:
			if (!strcmp("DmxStep_List_File", sexpr_text(fd))) {
				state = 1;
			} else {
				sexpr_close(fd);
			}
			break;
		case 1:
			if (sexpr_depth(fd) > 2) break;
			if (sexpr_depth(fd) < 2) {
				sexpr_close(fd);
				break;
			} 
			if (!strcmp("version", sexpr_text(fd))) {
				state = 11;
			} else if (!strcmp("time_length", sexpr_text(fd))) {
				state = 12;
			} else if (!strcmp("nb_cue", sexpr_text(fd))) {
				state = 14;
			} else if (!strcmp("cue", sexpr_text(fd))) {
				state = 15;
			}
			break;
		case 11:
			state = 1;
			if (sexpr_closed(fd) || sexpr_depth(fd) != 2) { 
				no_read = 1; 
				break; 
			}
			break;
		case 12:
			state = 1;
			if (sexpr_closed(fd) || sexpr_depth(fd) != 2) { 
				no_read = 1; 
				break; 
			}
			time_length = atol(sexpr_text(fd));
			break;

		case 14:
			state = 1;
			if (sexpr_closed(fd) || sexpr_depth(fd) != 2) { 
				no_read = 1; 
				break; 
			}
			i = atol(sexpr_text(fd));
			if (i < 0 || i > 2048) i = 16;
			if (i >= ma_cue) {
				cue = (ListAction**) realloc(cue, sizeof(ListAction*) * i);
				for (; ma_cue < i; ma_cue++) {
					cue[ma_cue] = NULL;
				}
			}
			break;

		case 15:
			state = 151;
			// FALLTHROUGH
		case 151:
			if (sexpr_depth(fd) != 3) { 
				state = 1;
				no_read = 1; 
				break; 
			}
			
			if (sexpr_element_id(fd, sexpr_depth(fd)) == 0) {
				replace(
					sexpr_element_id(fd, 
						sexpr_depth(fd) - 1) - 1, 
					sexpr_element_id(fd, sexpr_depth(fd)),
					sexpr_text(fd));
			}
		
			break;
		default:
			state = 0;
			no_read = 1; 
			break;
		}
	}

	changed = 0;
	Gui::unlock();
	return sexpr_close(fd);
}

int List::save(const char *file)
{
	int i;
	FILE *fd;
	char buf[2048];
	snprintf(buf, 2047, "%s/cue/", gui->work_dir);
	work_dir_len = strlen(buf);

	if (file) {
		free(file_name);
		file_name = strdup(file);
	}

	if (!file_name) return -1;

	fl_make_path_for_file(file_name);

	fd = fl_fopen(file_name, "w");
	if (!fd) {
		return -2;
	}

	fprintf(fd, "(DmxStep_List_File\n");
	fprintf(fd, "  (version  1.0.0)\n");
	fprintf(fd, "  (time_length  %d)\n", time_length);

	fprintf(fd, "  (nb_cue  %d)\n", nb_cue);

	fprintf(fd, "  (cue\n");
	for (i = 0; i < nb_cue; i++) {		
		if (cue[i] && cue[i]->cue && cue[i]->cue->file_name &&
			(int)strlen(cue[i]->cue->file_name) > work_dir_len) 
		{
			sexpr_quote(cue[i]->cue->file_name + work_dir_len, 
				strlen(cue[i]->cue->file_name) - work_dir_len, 
				buf, 2048);
			fprintf(fd, "    (%s)\n", buf);
		} else {
			fprintf(fd, "    ( )\n");
		}

	}
	fprintf(fd, "  )\n");

	fprintf(fd, ")\n");

	fclose(fd);

	changed = 0;

	

	return 0;
}

void List::do_step(int t)
{
	int i;
	if (t > time_length) t = time_length;
	if (t < 0) t = 0;
	for (i = 0; i < nb_cue; i++) {
		if (cue[i]) {
			int l = cue[i]->live;
			cue[i]->live = live;
			cue[i]->do_step(t);
			cue[i]->live = l;
		}
	}
}

char *List::get_value(int R, int C)
{
	static char buf[25];
	if (R < 0) return "";
/*
	switch (C) {
	case 0:
		break;
	case 1:
		if (R < nb_cue && cue[R] && cue[R]->file_name) {
			return  cue[R]->file_name + work_dir_len;
		}
		break;
	case 2:
		if (R < nb_cue && cue[R] && cue[R]->file_name) {
			snprintf(buf, 24, "%d", cue[R]->time_length / 100);
			return  buf;
		}
		break;
	case 3:
		if (R < nb_cue && cue[R] && cue[R]->file_name) {
			snprintf(buf, 24, "%d.%02d", cue[R]->bpm / 100,
				cue[R]->bpm % 100);
			return  buf;
		}
		break;
	}
*/
	return "";
}

int List::replace(int R, int C, const char *cu)
{
	char buf[2048];
	int ret = 0;
	int i;

	Gui::lock();

	if (R < 0) return -1;
	if (R >= ma_cue - 1) {
		cue = (ListAction**) realloc(cue, sizeof(ListAction*) * (R + 1));
		for (; ma_cue < (R + 1); ma_cue++) {
			cue[ma_cue] = NULL;
		}
	}
	if (!cu) {
		cue[R] = NULL;	
	} else {
		snprintf(buf, 2047, "%s/cue/", gui->work_dir);
		work_dir_len = strlen(buf);
		snprintf(buf, 2047, "%s/cue/%s",gui->work_dir, cu);
		//cue[R] = gui->cue_edit->get(buf);
		if (!cue[R]) ret = -1;
	}
	nb_cue = ma_cue;
	while (nb_cue > 0 && !cue[nb_cue - 1]) { nb_cue--;}

	time_length = 0;
	for (i = 0; i < nb_cue; i++) {
		if (cue[i]) {
		//	int l;
		//	l = cue[i]->time_length;
		//	if (l > time_length) time_length = l;
		}
	}
	changed = 1;

	Gui::unlock();
	return ret;
}



//
// End of "$Id: $".
//

