//
// "$Id: $"
//
//
//                   Copyright (c) 2003, O'ksi'D
//                      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of O'ksi'D nor the names of its contributors
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//

#include "ListEdit.h"
#include "Gui.h"
#include <FL/Fl_File_Chooser.H>

#ifdef WIN32
#define snprintf _snprintf
#endif

extern Gui *gui;

Fl_Menu_Item ListEdit::action_item[] = {
	{"None"},
	{"Play cue"},
	{"Pause cue"},
	{"Resume cue"},
	{"Stop cue"},
	{"Set cue begin step"},
	{"Set cue end step"},
	{"Set cue # loops"},
	{"Set cue clock source"},
	{"Set cue clock factor"},
	{"Blackout on!"},
	{"Blackout off"},
	{"Goto line"},
	{"Skip if end of cue"},
	{"Skip if increment >="},
	{"Skip if time >="},
	{"Skip if time <"},
	{"Wait for time >="},
	{"Wait for end of cue"},
	{"Run clock"},
	{"Stop clock"},
	{"Set clock BPM"},
	{"Set clock value"},
	{"Increment clock"},
	{"Execute program"},
	{"Send Telnet command"},
	{"Watch Telnet server"},
	{"Stop watch Telnet"},
	{"Set channel value"},
	{0}
};

static void list_sel_but_cb(Fl_Widget *w, void *d)
{
	((ListEdit*) d)->open_list(NULL);
}
 
static void list_save_but_cb(Fl_Widget *w, void *d)
{
	((ListEdit*) d)->save_list(((ListEdit*) d)->data->file_name);
}

static void list_saveas_but_cb(Fl_Widget *w, void *d)
{
	((ListEdit*) d)->save_list(NULL);
}

static void list_new_but_cb(Fl_Widget *w, void *d)
{
	((ListEdit*) d)->new_list();
}

static void list_step_minus_cb(Fl_Widget *w, void *d)
{
	char buf[256];
	ListEdit *s = (ListEdit*) d; 
	s->step -= 100 + s->step % 100;
	if (s->step < 0) s->step = 0;
	snprintf(buf, 256, "%05d.%02d", s->step / 100, s->step % 100);
	s->list_step->value(buf); 

	s->need_sync_output = 1;
}

static void list_step_plus_cb(Fl_Widget *w, void *d)
{
	char buf[256];
	ListEdit *s = (ListEdit*) d; 
	s->step += 100 - s->step % 100;
	if (s->step < 0) s->step = 0;
	snprintf(buf, 256, "%05d.%02d", s->step / 100, s->step % 100);
	s->list_step->value(buf); 
	s->need_sync_output = 1;
}

static void list_step_cb(Fl_Widget *w, void *d)
{
	char buf[256];
	ListEdit *s = (ListEdit*) d; 
	s->step = (int)(atof(s->list_step->value()) * 100.0);
	s->step -= s->step % 100;
	if (s->step < 0) s->step = 0;
	if (s->step >= s->data->time_length) {
		s->step = s->data->time_length - 100;
	}

	snprintf(buf, 256, "%05d.%02d", s->step / 100, s->step % 100);
	s->list_step->value(buf); 

	s->need_sync_output = 1;
}

static void list_begin_cb(Fl_Widget *w, void *d)
{
	char buf[256];
	ListEdit *s = (ListEdit*) d; 
	s->step = 0;
	snprintf(buf, 256, "%05d.%02d", s->step / 100, s->step % 100);
	s->list_step->value(buf); 
	s->need_sync_output = 1;
	s->start_10ms = s->last_10ms + 1;

}

static void list_end_cb(Fl_Widget *w, void *d)
{
	char buf[256];
	ListEdit *s = (ListEdit*) d; 
	

	s->step = s->data->time_length - 100;
	if (s->step < 0) s->step = 0;
	snprintf(buf, 256, "%05d.%02d", s->step / 100, s->step % 100);
	s->list_step->value(buf); 

	s->start_10ms = s->last_10ms + 1 - s->data->time_length;
	s->need_sync_output = 1;
}

static void list_stop_cb(Fl_Widget *w, void *d)
{
	ListEdit *s = (ListEdit*) d; 

	s->list_play->value(0);
	s->play = 0;
}

static void list_play_cb(Fl_Widget *w, void *d)
{
	ListEdit *s = (ListEdit*) d; 

	s->start_10ms = s->last_10ms + 1 - s->step;
	s->list_play->value(1);
	s->play = 1;
}

static void list_bpm_cb(Fl_Widget *w, void *d)
{
	char buf[256];
	ListEdit *s = (ListEdit*) d; 
	
	s->data->bpm = (int)(atof(s->list_bpm->value()) * 100.0);
	if (s->data->bpm< 0) s->data->bpm = 0;
	snprintf(buf, 256, "%d.%02d", s->data->bpm / 100, s->data->bpm % 100);
	s->list_bpm->value(buf); 
}

static void list_live_cb(Fl_Widget *w, void *d)
{
	ListEdit *s = (ListEdit*) d; 
	
	s->data->live = s->list_live->value();
}

static void list_loop_cb(Fl_Widget *w, void *d)
{
	//ListEdit *s = (ListEdit*) d; 
	
}

//static void list_export_cb(Fl_Widget *w, void *d)
//{
	//ListEdit *s = (ListEdit*) d; 
	
	// TODO
//}



ListEdit::ListEdit(int X, int Y, int W, int H, const char *T) :
	Fl_Group(X, Y, W, H, T)
{
	step = 0;
	data = new List();

	need_reload = 0;
	list_pool = NULL;
	ma_pool = 0;
	nb_pool = 0;

	last_10ms = 0;
	
	play = 0;

	list_file_inp = new Fl_Input(X + 5, Y + 4, 400, 25);
	list_file_inp->value("default.lst");

	list_new_but = new Fl_Button(X + 410, Y + 4, 40, 25, "new");
	list_new_but->callback(list_new_but_cb, this);
	list_sel_but = new Fl_Button(X + 455, Y + 4, 40, 25, "open");
	list_sel_but->callback(list_sel_but_cb, this);
	list_save_but = new Fl_Button(X + 500, Y + 4, 40, 25, "save");
	list_save_but->callback(list_save_but_cb, this);
	list_saveas_but = new Fl_Button(X + 545, Y + 4, 60, 25, "save as");
	list_saveas_but->callback(list_saveas_but_cb, this);


	list_step = new Fl_Float_Input(X + 135, Y + 32, 80, 25, "Running time");
	list_step->callback(list_step_cb, this);
	list_step->when(FL_WHEN_ENTER_KEY | FL_WHEN_RELEASE );
	list_step->value("00000.00");
	
	list_step_plus = new Fl_Button(X + 216, Y + 32, 12, 12, "+");
	list_step_plus->callback(list_step_plus_cb, this);
	list_step_minus = new Fl_Button(X + 216, Y + 43, 12, 12, "-");
	list_step_minus->callback(list_step_minus_cb, this);
/*
	list_length = new Fl_Output(X + 178, Y + 32, 80, 25, "Length");
	list_length->color(FL_GRAY);
	list_length->value("0.00");

	list_bpm = new Fl_Float_Input(X + 292, Y + 32, 50, 25, "BPM");
	list_bpm->when(FL_WHEN_ENTER_KEY | FL_WHEN_RELEASE );
	list_bpm->callback(list_bpm_cb, this);
	list_bpm->value("60.00");
*/
	list_live = new Fl_Button(X + 350, Y + 32, 25, 25, "Li");
	list_live->type(FL_TOGGLE_BUTTON);
	list_live->callback(list_live_cb, this);
	list_live->value(1);
/*
	list_loop = new Fl_Button(X + 380, Y + 32, 25, 25, "Lo");
	list_loop->type(FL_TOGGLE_BUTTON);
	list_loop->value(1);
	list_loop->callback(list_loop_cb, this);
*/
	list_begin = new Fl_Button(X + 410, Y + 32, 25, 25, "<<");
	list_begin->callback(list_begin_cb, this);
/*
	list_end = new Fl_Button(X + 438, Y + 32, 25, 25, ">>");
	list_end->callback(list_end_cb, this);
*/
	list_play = new Fl_Button(X + 470, Y + 32, 25, 25, "Pl");
	//list_play->type(FL_TOGGLE_BUTTON);
	list_play->callback(list_play_cb, this);
	list_play->when(FL_WHEN_RELEASE);
	list_play->value(0);

	list_stop = new Fl_Button(X + 498, Y + 32, 25, 25, "St");
	list_stop->callback(list_stop_cb, this);

//	list_export = new Fl_Button(X + 530, Y + 32, 50, 25, "Export");
//	list_export->callback(list_export_cb, this);

	list_table = new ListTable(X, Y + 60, W, H - 60, this);


	res = new Fl_Box( X + W - 2, Y + H - 2, 2, 2);
	res->hide();
	resizable(res);

	hide();
	end();
}


void ListEdit::save_list(const char *file)
{
	char buf[2048];
	int dl;

	snprintf(buf, 2047, "%s/list/", gui->work_dir);
	dl = strlen(buf);
	fl_make_path(buf);
	
	
	if (!file) 
	{
	
		snprintf(buf, 2047, "%s/list/%s", gui->work_dir,
				list_file_inp->value());

		Fl_File_Chooser fc(buf, 
			"Cue List (*.lst)", Fl_File_Chooser::CREATE, 
			"Save as cue list ...");
		fc.show();
		while (fc.visible())
			Fl::wait();
	
		if (fc.count() > 0 && fc.value(0) && !strncmp(fc.value(0), buf, dl)) {
			list_file_inp->value(fc.value(0) + dl);
		} else {
			if (fc.count() > 0) fl_alert("Unable to save to file!");
			return;
		} 
	} else {
		if (!strncmp(file, buf, dl)) {
			list_file_inp->value(file + dl);
		} else {
			fl_alert("Unable to save file!");
			return;
		}	
	}

	dl = strlen(list_file_inp->value());
	if (dl > 3 && !strcmp(list_file_inp->value() + dl - 4, ".lst")) {
		snprintf(buf, 2047, "%s/list/%s", gui->work_dir, 
			list_file_inp->value());
	} else {
		snprintf(buf, 2047, "%s/list/%s.lst", gui->work_dir, 
			list_file_inp->value());
	}
	fl_make_path_for_file(buf);	
	if (data->save(buf)) fl_alert("Unable to save file!");
	need_reload = 1;
	get(buf);
}


void ListEdit::new_list()
{
	Gui::lock();
	play = 0;
	list_play->value(0);
	if (data->changed) {
		if (fl_ask("Save changes ?")) save_list(data->file_name);
	}
	redraw();
	delete (data);
	data = new List();

	list_file_inp->value("default.lst");
	step = 0;
	list_step->value("00000.00");

	redraw();
	Gui::unlock();
}

void ListEdit::open_list(const char *file)
{
	char buf[2048];
	int dl;

	Gui::lock();

	if (data->changed) {
		if (fl_ask("Save changes ?")) save_list();
	}	

	snprintf(buf, 2047, "%s/list/", gui->work_dir);
	dl = strlen(buf);
	fl_make_path(buf);


	if (!file) {
		Fl_File_Chooser fc(buf, 
			"Cue List (*.lst)", Fl_File_Chooser::CREATE, 
			"Open cue list ...");
		fc.show();
		while (fc.visible())
			Fl::wait();
	
		if (fc.count() > 0 && fc.value(0) && !strncmp(fc.value(0), buf, dl)) {
			list_file_inp->value(fc.value(0) + dl);		
		} else {
			if (fc.count() > 0) fl_alert("Unable to open file!");
			return;
		} 
	} else {
		if (!strncmp(file, buf, dl)) {
			list_file_inp->value(file + dl);
		} else {
			fl_alert("Unable to open file!");
			return;
		}	
	}

	delete (data);
	data = new List();

	snprintf(buf, 2047, "%s/list/%s", gui->work_dir, list_file_inp->value());
	
	if (data->load(buf)) {
		fl_alert("Unable to open file!");
	}

	step = 0;
	list_step->value("00000.00");

	list_table->rows(data->nb_cue + 8);

	redraw();
	Gui::unlock();
}

void ListEdit::it_s_time_to_do_it(int time_10ms)
{

	last_10ms = time_10ms;

	if (play) {
		char buf[256];
		step = time_10ms - start_10ms;
		if (step < 0) step = 0;
		snprintf(buf, 256, "%05d.%02d", step / 100, step % 100);
		list_step->value(buf);
		Fl::awake();
		if (need_sync_output) need_sync_output = 0;
		data->do_step(step);
	} else 	if (need_sync_output) {
		need_sync_output = 0;
		data->do_step(step);
		Fl::awake();
	}
	
}



int ListEdit::handle(int e)
{
	if (e == FL_SHORTCUT && Fl::event_length() == 1) {
		switch (Fl::event_text()[0]) {
		case '+':
			list_step_plus->do_callback();
			return 1;
		case '-':
			list_step_minus->do_callback();
			return 1;
		case ' ':
			if (play) {
				list_stop->do_callback();
			} else {
				list_play->do_callback();
			}
			return 1;
		}	
	}
	return Fl_Group::handle(e);
}

char *ListEdit::get_value(int R, int C)
{
	return data->get_value(R, C);

}
void ListEdit::set_value(int row_edit, int col_edit)
{
	Gui::lock();
	
//	char buf[2048];
//	int dl;


	data->replace(row_edit, col_edit, NULL);
/*
	if (col_edit == 0) {
		data->replace(row_edit, col_edit, NULL);
	} else if (col_edit == 1) {
		snprintf(buf, 2047, "%s/cue/", gui->work_dir);
		dl = strlen(buf);
		fl_make_path(buf);

		Fl_File_Chooser fc(buf, 
			"Cue (*.cue)", Fl_File_Chooser::CREATE, 
			"Open Cue ...");
		fc.show();
		while (fc.visible())
			Fl::wait();
	
		if (fc.count() > 0 && fc.value(0) && !strncmp(fc.value(0), buf, dl)) {
			data->replace(row_edit, fc.value(0) + dl);		
		} else {
			if (fc.count() > 0) fl_alert("Unable to open file!");
			return;
		} 
	}
*/
/*
	snprintf(buf, 24, "%d.%02d", data->time_length / 100,
		data->time_length % 100);
	list_length->value(buf);
*/

	Gui::unlock();
}

List *ListEdit::get(const char *seq)
{
	int i, f;

	Gui::lock();

	f = -1;
	for (i = 0; i < nb_pool; i++) {
		if (list_pool[i]) {
			if (list_pool[i]->file_name &&
				!strcmp(seq, list_pool[i]->file_name)) 
			{
				if (need_reload) list_pool[i]->load(seq);
				need_reload = 0;
				Gui::unlock();
				return list_pool[i];
			}
		} else {
			f = i;
		}
	}

	if (f == -1) {
		f = nb_pool++;
		if (f >= ma_pool) {
			int i;
			ma_pool += 32;
			list_pool = (List**) realloc(list_pool,
				sizeof(List*) * (ma_pool));
			for (i = f; i < ma_pool; i++) {
				list_pool[i] = NULL;
			}
		}
	}

	list_pool[f] = new List();
	list_pool[f]->load(seq);
	Gui::unlock();
	return list_pool[f];
}

//
// End of "$Id: $".
//

