//
// "$Id: $"
//
//
//                   Copyright (c) 2003-2004, O'ksi'D
//                      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of O'ksi'D nor the names of its contributors
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//

#include "CueEdit.h"
#include "Gui.h"
#include "sexpr.h"
#include <FL/Fl_Window.H>
#include <FL/Fl.H>
#include <FL/fl_ask.H>
#include <FL/fl_draw.H>
#include <FL/Fl_Preferences.H>
#include <stdlib.h>
#include <stdio.h>
#ifdef WIN32
#define snprintf _snprintf
#define S_IRGRP 0
#define S_IROTH 0
#include <io.h>
#else
#include <unistd.h>
#endif
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <FL/Fl_File_Chooser.H>

extern Gui *gui;

ListTable::ListTable(int x, int y, int w, int h, ListEdit *d) : 
	Fl_Table(x, y, w, h)
{
	list_edit = d;

	callback(&event_callback, (void*)this);
	when(FL_WHEN_NOT_CHANGED|when()|FL_WHEN_RELEASE);

	input = new ListInput(w/2,h/2,0,0);
	input->hide();

	(new Fl_Box(9999,9999,0,0))->hide();  // HACK: prevent flickering in Fl_Scroll

	cols(10);
	rows(11);
	col_header(1);
	col_resize(1);
	row_header(1);
	row_resize(1);
	row_height_all(18);
	col_width_all(75);

	col_header_height(35);
	col_width(0, 30);
	col_width(1, 150);
	col_width(3, 250);
	
	row_header_width(25);

	end();
}


// Callback whenever someone clicks on different parts of the table
void ListTable::event_callback(Fl_Widget*, void *data)
{
    ListTable *o = (ListTable*)data;
    o->event_callback2();
}

void ListTable::event_callback2()
{
    int R = callback_row(),
        C = callback_col();
    TableContext context = callback_context();
    int XX,YY,WW,HH;


    switch ( context )
    {
	case CONTEXT_CELL:
	{
            switch (Fl::event()) 
	    { 
		case FL_PUSH:
		    Fl::event_clicks(0);
		case FL_KEYBOARD:
		    if (Fl::event() == FL_KEYBOARD && Fl::e_length == 0) { 
			return; 
		    }
		    do_cb();
		    hide_inp();

		    if (R > rows() - 2) rows(rows() + 8);

		    row_edit = R;
		    col_edit = C;
		    set_selection(R, C, R, C);
		    
		    find_cell(CONTEXT_CELL, R, C, XX, YY, WW, HH);
		    
		    input->resize(XX,YY,WW,HH);
		    input->data(R, C, this);
		    input->show();
		    input->take_focus();
		
		    if (Fl::event() == FL_KEYBOARD && Fl::e_text[0] != '\r') { 
			input->handle(Fl::event()); 
		    }
		    return;
	    }
	    return;
	}


	case CONTEXT_ROW_HEADER:
	case CONTEXT_COL_HEADER:
	    do_cb();
	    hide_inp();
	    return;

	case CONTEXT_TABLE:
	    if (R < 0 && C < 0)
	    {
		do_cb();
		hide_inp();
	    }
	    return;
	
	default:
	    return;
    }
}

void ListTable::do_cb()
{
	if (input->visible()) input->do_callback();

}

void ListTable::hide_inp()
{
	input->hide();
}

// Handle drawing all cells in table
void ListTable::draw_cell(TableContext context, 
			    int R, int C, int X, int Y, int W, int H)
{
    static char s[30];
    if (input->visible() && !input->contains(Fl::focus())) { 
	    do_cb();
	    hide_inp();
    }
    switch ( context )
    {
	case CONTEXT_STARTPAGE:
	    // Get kb nav + mouse 'selection region' for use below
	    get_selection(s_top, s_left, s_bottom, s_right);	
	    break;

	case CONTEXT_COL_HEADER:
	    fl_font(FL_HELVETICA | FL_BOLD, 14);
	    fl_push_clip(X, Y, W, H);
	    {
		fl_draw_box(FL_THIN_UP_BOX, X, Y, W, H, col_header_color());
		fl_color(FL_BLACK);
	        fl_draw(list_edit->get_value(-1, C), 
			X + 3, Y + 3, W - 6, H - 6, FL_ALIGN_CENTER);
	    }
	    fl_pop_clip();
	    return;

	case CONTEXT_ROW_HEADER:
	    fl_font(FL_HELVETICA | FL_BOLD, 14);
	    fl_push_clip(X, Y, W, H);
	    {
		char buf[25];
		snprintf(buf, 24, "%02d", R);
		fl_draw_box(FL_THIN_UP_BOX, X, Y, W, H, row_header_color());
		fl_color(FL_BLACK);
		fl_draw(buf, X, Y, W, H, FL_ALIGN_CENTER);
	    }
	    fl_pop_clip();
	    return;

	case CONTEXT_CELL:
	{
	    if (R == row_edit && C == col_edit && 
		(input->visible()))
	    { 
		return; 
	    }

	    // BACKGROUND
	    fl_push_clip(X, Y, W, H);

	    // Keyboard nav and mouse selection highlighting

	    if (R >= s_top && R <= s_bottom && C >= s_left && C <= s_right) {
		fl_draw_box(FL_THIN_UP_BOX, X, Y, W, H, FL_YELLOW);
            } else {
	        fl_draw_box(FL_THIN_UP_BOX, X, Y, W, H, FL_WHITE); 
	    }


 
	    fl_pop_clip();
	    // TEXT
	    fl_push_clip(X+3, Y+3, W-6, H-6);
	    {
		fl_font(FL_HELVETICA, 14);
		fl_color(FL_BLACK);
	        fl_draw(list_edit->get_value(R, C), 
			X + 3, Y + 3, W - 6, H - 6, FL_ALIGN_LEFT);
	    }
	    fl_pop_clip();
	    return;
	}

	case CONTEXT_RC_RESIZE:
	{
	    if (!input->visible()) return;
	    find_cell(CONTEXT_TABLE, row_edit, col_edit, X, Y, W, H);
	    input->resize(X,Y,W,H);
	    return;
	}
	default:
	    return;
    }
}


//
// End of "$Id: $".
//

