//
// "$Id: $"
//
//
//                   Copyright (c) 2003, O'ksi'D
//                      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of O'ksi'D nor the names of its contributors
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//

#include "Scene.h"
#include "Gui.h"
#include <stdlib.h>
#include <stdio.h>
#include <FL/Fl.H>
#ifdef WIN32
#include <io.h>
#define usleep(x) Sleep((x) / 1000)
int gettimeofday(struct timeval *tp, void *tzp);
#else
#include <unistd.h>
#endif
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

extern Gui *gui;

Scene::Scene(int X, int Y, int W, int H, const char *T) :
	Fl_Group(X, Y, W, H, T)
{
	InitializeCriticalSection(&cs);
	current_div = 0;
	current_step = 0;

	set_bpm(60, 50);

	
	hide();
	end();
}

void Scene::step(void)
{
	lock();
	gui->list_edit->it_s_time_to_do_it(current_step * 600 + current_div * 10);
	gui->cue_edit->it_s_time_to_do_it(current_step * 600 + current_div * 10);
	gui->sequence->it_s_time_to_do_it(current_step * 600 + current_div * 10);
	
	current_div++;
	if (!(current_div % clock_divider)) {
		current_div = 0;
		current_step++;
		printf("step %d %d\n", current_step, clock_divider);
	}
	unlock();
}

void Scene::unlock() {
#ifdef WIN32
	LeaveCriticalSection(&cs);
#endif
}


void Scene::lock() {
#ifdef WIN32
	EnterCriticalSection(&cs);
#endif
}


void Scene::set_bpm(double b, int r)
{
	double bb;

	bpm = (int) b;
	bb = 1.0 / (b / 60.0) / r;
	timeout.tv_sec = (long)bb;
	timeout.tv_usec = (long) ((bb - timeout.tv_sec) * 1000000);
	clock_divider = r;

	if (timeout.tv_sec == 0 && timeout.tv_usec < 20000 && r > 1) {
		int d;
		d = 20000 / timeout.tv_usec;
		if (d < 2) d = 2;
		clock_divider /= d;		
	}
}


void *scene_thread(void *data)
{
	struct timeval tv, tm, ma;
	Gui *gui = (Gui*) data;

	gettimeofday(&tm, NULL);
	tv.tv_sec = tm.tv_sec;
	tv.tv_usec = tm.tv_usec;
	while (1) {
		int go = 0;
		do {
			gui->scene->step();
			tv.tv_sec += gui->scene->timeout.tv_sec;
			tv.tv_usec += gui->scene->timeout.tv_usec;
			if (tv.tv_usec >= 1000000) {
				tv.tv_usec -= 1000000;
				tv.tv_sec++;
			}
			//printf("%ld %ld %ld\n", tm.tv_sec, tv.tv_sec, tm.tv_usec);
			go++;
		} while (!(tv.tv_sec > tm.tv_sec || 
			(tv.tv_sec == tm.tv_sec && tv.tv_usec >= tm.tv_usec)));

		if (go > 1) printf ("slice %ld\n", go);
		gettimeofday(&tm, NULL);
		ma.tv_sec = tv.tv_sec;
		ma.tv_usec = tv.tv_usec - 10000;
		if (ma.tv_usec < 0) {
			ma.tv_usec += 1000000;
			ma.tv_sec--;
		} 
		while (ma.tv_sec > tm.tv_sec || 
			(ma.tv_sec == tm.tv_sec && ma.tv_usec >= tm.tv_usec)) 
		{
			struct timeval s;
			s.tv_sec = tv.tv_sec - tm.tv_sec;
			s.tv_usec = tv.tv_usec - tm.tv_usec;
			s.tv_usec -= 5000;
			if (s.tv_usec < 0) {
				s.tv_usec += 1000000;
				s.tv_sec--;
			}
#ifdef WIN32			
			SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_HIGHEST);
			Sleep(s.tv_usec / 1000);
			SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_NORMAL);
#else
			select(0, NULL, NULL, NULL, &s);
#endif
			gettimeofday(&tm, NULL);
		}
	}
	return 0;
}

//
// End of "$Id: $".
//

