//
// "$Id: $"
//
//
//                   Copyright (c) 2003, O'ksi'D
//                      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of O'ksi'D nor the names of its contributors
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//

#include "Sequence.h"
#include "Gui.h"
#include <FL/Fl_File_Chooser.H>

#ifdef WIN32
#define snprintf _snprintf
#endif

extern Gui *gui;

static void seq_sel_but_cb(Fl_Widget *w, void *d)
{
	((Sequence*) d)->open_seq();
}
 
static void seq_save_but_cb(Fl_Widget *w, void *d)
{
	((Sequence*) d)->save_seq();
}

static void seq_saveas_but_cb(Fl_Widget *w, void *d)
{
	((Sequence*) d)->saveas_seq();
}

static void seq_new_but_cb(Fl_Widget *w, void *d)
{
	((Sequence*) d)->new_seq();
}

static void seq_step_minus_cb(Fl_Widget *w, void *d)
{
	char buf[256];
	Sequence *s = (Sequence*) d; 
	s->step -= 100 + s->step % 100;
	if (s->step < 0) s->step = 0;
	snprintf(buf, 256, "%05d.%02d", s->step / 100, s->step % 100);
	s->seq_step->value(buf); 
	if (s->step >= s->data->time_length) {
		s->data->time_length = s->step + 100;
		if (s->step % 100)  s->data->time_length -=  (s->step % 100);
		snprintf(buf, 256, "%d", s->data->time_length / 100);
		s->seq_length->value(buf);
	}
	s->need_sync_output = 1;
}

static void seq_step_plus_cb(Fl_Widget *w, void *d)
{
	char buf[256];
	Sequence *s = (Sequence*) d; 
	s->step += 100 - s->step % 100;
	if (s->step < 0) s->step = 0;
	snprintf(buf, 256, "%05d.%02d", s->step / 100, s->step % 100);
	s->seq_step->value(buf); 
	if (s->step >= s->data->time_length) {
		s->data->time_length = s->step + 100;
		if (s->step % 100)  s->data->time_length -= (s->step % 100);
		snprintf(buf, 256, "%d", s->data->time_length / 100);
		s->seq_length->value(buf);
	}
	s->need_sync_output = 1;
}

static void seq_step_cb(Fl_Widget *w, void *d)
{
	char buf[256];
	Sequence *s = (Sequence*) d; 
	s->step = (int)(atof(s->seq_step->value()) * 100.0);
	if (s->step < 0) s->step = 0;
	snprintf(buf, 256, "%05d.%02d", s->step / 100, s->step % 100);
	s->seq_step->value(buf); 
	if (s->step >= s->data->time_length) {
		s->data->time_length = s->step + 100;
		if (s->step % 100)  s->data->time_length -=  (s->step % 100);
		snprintf(buf, 256, "%d", s->data->time_length / 100);
		s->seq_length->value(buf);
	}
	s->need_sync_output = 1;
}

static void seq_length_cb(Fl_Widget *w, void *d)
{
	char buf[256];
	Sequence *s = (Sequence*) d; 
	s->data->time_length = atol(s->seq_length->value()) * 100;
	if (s->data->time_length < 100) s->data->time_length = 100;
	snprintf(buf, 256, "%d", s->data->time_length / 100);
	s->seq_length->value(buf);
}

static void seq_begin_cb(Fl_Widget *w, void *d)
{
	char buf[256];
	Sequence *s = (Sequence*) d; 
	s->step = 0;
	if (s->step < 0) s->step = 0;
	snprintf(buf, 256, "%05d.%02d", s->step / 100, s->step % 100);
	s->seq_step->value(buf); 
	s->need_sync_output = 1;
	s->start_10ms = s->last_10ms + 1;

}

static void seq_end_cb(Fl_Widget *w, void *d)
{
	char buf[256];
	Sequence *s = (Sequence*) d; 
	
	if (s->seq_low_res->value()) {
		s->step = s->data->time_length - 100;
	} else {
		s->step = s->data->time_length - 1;
	} 
	if (s->step < 0) s->step = 0;
	snprintf(buf, 256, "%05d.%02d", s->step / 100, s->step % 100);
	s->seq_step->value(buf); 
	if (s->step >= s->data->time_length) {
		s->data->time_length = s->step + 100;
		if (s->step % 100)  s->data->time_length -=  (s->step % 100);
		snprintf(buf, 256, "%d", s->data->time_length / 100);
		s->seq_length->value(buf);
	}
	s->start_10ms = s->last_10ms + 1 - (int)(s->data->time_length / (s->data->bpm / 6000.0));
	s->need_sync_output = 1;
}

static void seq_stop_cb(Fl_Widget *w, void *d)
{
	Sequence *s = (Sequence*) d; 

	s->seq_play->value(0);
	s->play = 0;
}

static void seq_play_cb(Fl_Widget *w, void *d)
{
	Sequence *s = (Sequence*) d; 

	s->start_10ms = s->last_10ms + 1 - (int)(s->step / (s->data->bpm / 6000.0));
	s->seq_play->value(1);
	s->play = 1;
}

static void seq_bpm_cb(Fl_Widget *w, void *d)
{
	char buf[256];
	Sequence *s = (Sequence*) d; 
	
	s->data->bpm = (int)(atof(s->seq_bpm->value()) * 100.0);
	if (s->data->bpm< 0) s->data->bpm = 0;
	snprintf(buf, 256, "%d.%02d", s->data->bpm / 100, s->data->bpm % 100);
	s->seq_bpm->value(buf); 
}

static void seq_live_cb(Fl_Widget *w, void *d)
{
	Sequence *s = (Sequence*) d; 
	
	s->data->live = s->seq_live->value();
}

static void seq_edit_cb(Fl_Widget *w, void *d)
{
	//Sequence *s = (Sequence*) d; 
	
	// TODO
}

static void fixture_cb(FixtureTree *f, void *d)
{
	((Sequence*)d)->fix_tree_cb();
}

Sequence::Sequence(int X, int Y, int W, int H, const char *T) :
	Fl_Group(X, Y, W, H, T)
{
	step = 0;
	data = new StepGroup();

	need_reload = 0;
	sequence_pool = NULL;
	ma_pool = 0;
	nb_pool = 0;

	grp = NULL;

	last_10ms = 0;
	
	play = 0;

	seq_file_inp = new Fl_Input(X + 5, Y + 4, 400, 25);
	seq_file_inp->value("default.seq");

	seq_new_but = new Fl_Button(X + 410, Y + 4, 40, 25, "new");
	seq_new_but->callback(seq_new_but_cb, this);
	seq_sel_but = new Fl_Button(X + 455, Y + 4, 40, 25, "open");
	seq_sel_but->callback(seq_sel_but_cb, this);
	seq_save_but = new Fl_Button(X + 500, Y + 4, 40, 25, "save");
	seq_save_but->callback(seq_save_but_cb, this);
	seq_saveas_but = new Fl_Button(X + 545, Y + 4, 60, 25, "save as");
	seq_saveas_but->callback(seq_saveas_but_cb, this);


	seq_step = new Fl_Float_Input(X + 35, Y + 32, 80, 25, "Step");
	seq_step->callback(seq_step_cb, this);
	seq_step->when(FL_WHEN_ENTER_KEY | FL_WHEN_RELEASE );
	seq_step->value("00000.00");
	
	seq_step_plus = new Fl_Button(X + 116, Y + 32, 12, 12, "+");
	seq_step_plus->callback(seq_step_plus_cb, this);
	seq_step_minus = new Fl_Button(X + 116, Y + 43, 12, 12, "-");
	seq_step_minus->callback(seq_step_minus_cb, this);

	seq_length = new Fl_Int_Input(X + 178, Y + 32, 50, 25, "Length");
	seq_length->when(FL_WHEN_ENTER_KEY | FL_WHEN_RELEASE );
	seq_length->callback(seq_length_cb, this);
	seq_length->value("16");


	seq_bpm = new Fl_Float_Input(X + 266, Y + 32, 50, 25, "BPM");
	seq_bpm->when(FL_WHEN_ENTER_KEY | FL_WHEN_RELEASE );
	seq_bpm->callback(seq_bpm_cb, this);
	seq_bpm->value("60.00");

	seq_low_res = new Fl_Button(X + 320, Y + 32, 25, 25, "loR");
	seq_low_res->type(FL_TOGGLE_BUTTON);
	seq_low_res->value(1);

	seq_live = new Fl_Button(X + 350, Y + 32, 25, 25, "Li");
	seq_live->type(FL_TOGGLE_BUTTON);
	seq_live->callback(seq_live_cb, this);
	seq_live->value(1);

	seq_loop = new Fl_Button(X + 380, Y + 32, 25, 25, "Lo");
	seq_loop->type(FL_TOGGLE_BUTTON);
	seq_loop->value(1);

	seq_begin = new Fl_Button(X + 410, Y + 32, 25, 25, "<<");
	seq_begin->callback(seq_begin_cb, this);

	seq_end = new Fl_Button(X + 438, Y + 32, 25, 25, ">>");
	seq_end->callback(seq_end_cb, this);

	seq_play = new Fl_Button(X + 470, Y + 32, 25, 25, "Pl");
	//seq_play->type(FL_TOGGLE_BUTTON);
	seq_play->callback(seq_play_cb, this);
	seq_play->when(FL_WHEN_RELEASE);
	seq_play->value(0);

	seq_stop = new Fl_Button(X + 498, Y + 32, 25, 25, "St");
	seq_stop->callback(seq_stop_cb, this);

	seq_edit = new Fl_Button(X + 530, Y + 32, 50, 25, "Edit");
	seq_edit->callback(seq_edit_cb, this);

	fix_tree = new FixtureTree(X + 5, Y + 60, 250, H - 65);
	fix_tree->set_cb(fixture_cb, this);

	slider_scroll = new Fl_Scroll(X + 260, Y + 60, 250, H - 65);
	slider_scroll->box(FL_FLAT_BOX);
	slider_scroll->end();

	res = new Fl_Box( X + W - 2, Y + H - 7, 2, 2);
	res->hide();
	resizable(res);

	hide();
	end();
}

Sequence::~Sequence()
{
	while (--nb_pool >= 0) {
		delete(sequence_pool[nb_pool]);
	}
	free(sequence_pool);
	
}

void Sequence::save_seq()
{
	char buf[2048];
	int dl;
	
	

	if (!seq_file_inp->value() || !seq_file_inp->value()[0] ||
		!strcmp(seq_file_inp->value(), "default.seq")) 
	{
		snprintf(buf, 2047, "%s/sequence/", gui->work_dir);
		dl = strlen(buf);

		Fl_File_Chooser fc(buf, 
			"Sequence (*.seq)", Fl_File_Chooser::CREATE, 
			"Save as sequence ...");
		fc.show();
		while (fc.visible())
			Fl::wait();
	
		if (fc.count() > 0 && fc.value(0) && !strncmp(fc.value(0), buf, dl)) {
			seq_file_inp->value(fc.value(0) + dl);
		} else {
			fl_alert("Unable to save to file!");
			return;
		} 
	} 
	dl = strlen(seq_file_inp->value());
	if (dl > 3 && !strcmp(seq_file_inp->value() + dl - 4, ".seq")) {
		snprintf(buf, 2047, "%s/sequence/%s", gui->work_dir, 
			seq_file_inp->value());
	} else {
		snprintf(buf, 2047, "%s/sequence/%s.seq", gui->work_dir, 
			seq_file_inp->value());
	}
	fl_make_path_for_file(buf);	
	if (data->save(buf)) fl_alert("Unable to save file!");
	
	need_reload = 1;
	get(buf);
}

void Sequence::saveas_seq()
{
	char buf[2048];
	int dl;

	
	{
		snprintf(buf, 2047, "%s/sequence/", gui->work_dir);
		dl = strlen(buf);
		snprintf(buf, 2047, "%s/sequence/%s", gui->work_dir, seq_file_inp->value());

		Fl_File_Chooser fc(buf, 
			"Sequence (*.seq)", Fl_File_Chooser::CREATE, 
			"Save as sequence ...");
		fc.show();
		while (fc.visible())
			Fl::wait();
	
		if (fc.count() > 0 && fc.value(0) && !strncmp(fc.value(0), buf, dl)) {
			seq_file_inp->value(fc.value(0) + dl);			
		} else {
			if (fc.count() > 0) fl_alert("Unable to save to file!");
			return;
		} 
	} 

	dl = strlen(seq_file_inp->value());
	if (dl > 3 && !strcmp(seq_file_inp->value() + dl - 4, ".seq")) {
		snprintf(buf, 2047, "%s/sequence/%s", gui->work_dir, 
			seq_file_inp->value());
	} else {
		snprintf(buf, 2047, "%s/sequence/%s.seq", gui->work_dir, 
			seq_file_inp->value());
	}
	fl_make_path_for_file(buf);
	if (data->save(buf)) fl_alert("Unable to save file!");
	
	need_reload = 1;
	get(buf);
}

void Sequence::new_seq()
{
	char buf[256];
	Gui::lock();
	play = 0;
	seq_play->value(0);
	if (data->changed) {
		if (fl_ask("Save changes ?")) save_seq();
	}
	slider_scroll->clear();
	grp = NULL;
	redraw();
	delete (data);
	data = new StepGroup();

	seq_file_inp->value("default.seq");
	need_sync_output = 1;
	snprintf(buf, 256, "%d", data->time_length / 100);
	seq_length->value(buf);
	step = 0;
	seq_step->value("00000.00");
	snprintf(buf, 256, "%d.%02d", data->bpm / 100, data->bpm % 100);
	seq_bpm->value(buf); 
	Gui::unlock();
}

void Sequence::open_seq()
{
	char buf[2048];
	int dl;
	
	Gui::lock();

	new_seq();

	{
		snprintf(buf, 2047, "%s/sequence/", gui->work_dir);
		dl = strlen(buf);
		fl_make_path(buf);

		Fl_File_Chooser fc(buf, 
			"Sequence (*.seq)", Fl_File_Chooser::CREATE, 
			"Open sequence ...");
		fc.show();
		while (fc.visible())
			Fl::wait();
	
		if (fc.count() > 0 && fc.value(0) && !strncmp(fc.value(0), buf, dl)) {
			seq_file_inp->value(fc.value(0) + dl);		
		} else {
			if (fc.count() > 0) fl_alert("Unable to open file!");
			Gui::unlock();
			return;
		} 
	} 

	snprintf(buf, 2047, "%s/sequence/%s", gui->work_dir, seq_file_inp->value());
	
	if (data->load(buf)) {
		fl_alert("Unable to open file!");
	}

	snprintf(buf, 256, "%d", data->time_length / 100);
	seq_length->value(buf);
	step = 0;
	seq_step->value("00000.00");
	snprintf(buf, 256, "%d.%02d", data->bpm / 100, data->bpm % 100);
	seq_bpm->value(buf); 

	need_sync_output = 1;	
	Gui::unlock();
}

void Sequence::it_s_time_to_do_it(int time_10ms)
{

	last_10ms = time_10ms;

	//printf("%d\n", gui->jump);
	if (play) {
		char buf[256];
		step = (int)((time_10ms - start_10ms) * (data->bpm / 6000.0));
		
		while (step >= data->time_length) {
			int n;
			if (!seq_loop->value()) {
				seq_play->value(0);
				play = 0;
				Fl::awake();
				return;
			}
			n = (int)(data->time_length / (data->bpm / 6000.0));
			if (n < 1) n = 1;
			start_10ms += n; 
			step = (int)((time_10ms - start_10ms) * (data->bpm / 6000.0));
		}
		if (step < 0) step = 0;
		if (seq_low_res->value()) {
			step -= step % 100;
		} 
		snprintf(buf, 256, "%05d.%02d", step / 100, step % 100);
		seq_step->value(buf);
		Fl::awake();
		if (need_sync_output) need_sync_output = 0;
		data->do_step(step);
		if (grp) grp->copy_from_steps(data);
	} else 	if (need_sync_output) {
		need_sync_output = 0;
		data->do_step(step);
		if (grp) grp->copy_from_steps(data);
		Fl::awake();
	}
	
}


static void panel_cb(FixtureTypeChannelGroup *g, void *data)
{
	Sequence *s = (Sequence*) data;
	g->copy_to_steps(s->step, s->data);
	s->need_sync_output = 1;
}

void Sequence::fix_tree_cb()
{
	int n = fix_tree->get_selection();
	if (fix_tree->item_flags(n) & Fl_Tree::FLAG_FOLDER) return;
	// FLAG_SELECTED
	n = (int) fix_tree->item_data(n);
	if (n >= gui->devices->nb_fix) return;

	slider_scroll->clear();
	slider_scroll->position(0, 0);
	grp = gui->devices->fix[n]->get_panel(panel_cb, this);
	grp->id(n);
	grp->copy_from_steps(data);


	slider_scroll->add(grp);
	grp->position(slider_scroll->x(), slider_scroll->y());
	slider_scroll->redraw();

}

int Sequence::handle(int e)
{
	if (e == FL_SHORTCUT && Fl::event_length() == 1) {
		switch (Fl::event_text()[0]) {
		case '+':
			seq_step_plus->do_callback();
			return 1;
		case '-':
			seq_step_minus->do_callback();
			return 1;
		case ' ':
			if (play) {
				seq_stop->do_callback();
			} else {
				seq_play->do_callback();
			}
			return 1;
		}	
	}
	return Fl_Group::handle(e);
}

StepGroup *Sequence::get(const char *seq)
{
	int i, f;
	Gui::lock();

	f = -1;
	for (i = 0; i < nb_pool; i++) {
		if (sequence_pool[i]) {
			if (sequence_pool[i]->file_name &&
				!strcmp(seq, sequence_pool[i]->file_name)) 
			{
				if (need_reload) sequence_pool[i]->load(seq);
				need_reload = 0;
				Gui::unlock();
				return sequence_pool[i];
			}
		} else {
			f = i;
		}
	}

	if (f == -1) {
		f = nb_pool++;
		if (f >= ma_pool) {
			int i;
			ma_pool += 32;
			sequence_pool = (StepGroup**) realloc(sequence_pool,
				sizeof(StepGroup*) * (ma_pool));
			for (i = f; i < ma_pool; i++) {
				sequence_pool[i] = NULL;
			}
		}
	}

	sequence_pool[f] = new StepGroup();
	sequence_pool[f]->load(seq);
	Gui::unlock();
	return sequence_pool[f];
}

//
// End of "$Id: $".
//

