//
// "$Id: $"
//
//
//                   Copyright (c) 2003-2004, O'ksi'D
//                      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of O'ksi'D nor the names of its contributors
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//

#include "Step.h"
#include "Gui.h"
#include "sexpr.h"

#include <stdlib.h>
#include <stdio.h>
#ifdef WIN32
#define snprintf _snprintf
#define S_IRGRP 0
#define S_IROTH 0
#include <io.h>
#else
#include <unistd.h>
#endif
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

extern Gui *gui;


Step::Step(int t)
{
	time = t;
	nb_action = 0;
	ma_action = 0;
	action = NULL;
}

Step::~Step()
{
	free(action);
}

void Step::del_action(int de, int ch)
{
	int i;
	int go = 0;
	for (i = 0; i < nb_action; i++) {
		if (!go && action[i].device == de && action[i].channel == ch) {
			go = 1;
			nb_action--;
		}
		if (go && i < nb_action) {
			memcpy(&action[i], &action[i+1], sizeof(Action));
/*
			action[i].device = action[i+1].device;
			action[i].channel = action[i+1].channel;
			action[i].curve = action[i+1].curve;
			action[i].value = action[i+1].value;
			action[i].next_value = action[i+1].next_value;
			action[i].next_time = action[i+1].next_time;	
*/
		}
	}
}

void Step::add_action(int de, int ch, int cu, int mi, int va, int nt, int nv)
{
	int i, n;
	if (nb_action >= ma_action) {
		ma_action += 16;
		action = (Action*) realloc(action, sizeof(Action) * ma_action);
	}

	n = nb_action;
	
	for (i = 0; i < nb_action; i++) {
		if (action[i].device == de && action[i].channel == ch) {
			n = i;
			break;
		}
	}
	action[n].time = time;
	action[n].device = de;
	action[n].channel = ch;
	action[n].curve = cu;
	action[n].mix_mode = mi;
	action[n].value = va;
	action[n].next_value = nv;
	action[n].next_time = nt;

	if (n == nb_action) nb_action++;
}


StepGroup::StepGroup()
{
	time_length = 1600;
	bpm = 6000;
	live = 1;

	divisor = bpm / 6000.0;

	file_name = NULL;
		
	ma_step = 0;
	nb_step = 0;
	step = NULL;

	ma_stack = 0;
	nb_stack = 0;
	stack = NULL;

	current = 0;

	changed = 0;

	dev = NULL;
}

StepGroup::~StepGroup()
{
	free(stack);
	free(file_name);
	while (nb_step > 0) {
		nb_step--;
		delete(step[nb_step]);
	}
	free(step);
	free(dev);
}

void StepGroup::del_action(int t, int de, int ch)
{
	int i;
	Action *a = NULL;
	int curr;

	Gui::lock();

	// delete the action
	for (i = 0; i < nb_step; i++) {
		if (step[i]->time == t) {
			step[i]->del_action(de, ch);
			break;
		}
	}

	// find the previous action which references our action
	for (i = 0; i < nb_step && !a; i++) {
		int j;
		Step *s = step[i];
		for (j = 0; j < s->nb_action; j++) {
			if (s->action[j].device == de && 
				s->action[j].channel == ch &&
				s->action[j].next_time == t) 
			{
				curr = i;
				a = &s->action[j];
				break;
			}
		}
	}

	if (!a) {
		Gui::unlock();
		return;
	}
	

	// ok, now update the reference
	a->next_time = -1;

	for (i = curr + 1; i < nb_step; i++) {
		int j;
		Step *s = step[i];
		for (j = 0; j < s->nb_action; j++) {
			if (s->action[j].device == de && 
				s->action[j].channel == ch) 
			{
				a->next_time = s->action[j].time;
				a->next_value = s->action[j].value;
				Gui::unlock();
				return;
			}
		}
	}

	for (i = 0; i < curr; i++) {
		int j;
		Step *s = step[i];
		for (j = 0; j < s->nb_action; j++) {
			if (s->action[j].device == de && 
				s->action[j].channel == ch) 
			{
				a->next_time = s->action[j].time;
				a->next_value = s->action[j].value;
				Gui::unlock();
				return;
			}
		}
	}		
	Gui::unlock();
}

void StepGroup::add_action(int t, int de, int ch, int cu, int mi, int va)
{
	int i;
	int nt = -1;
	int nv = va;
	
	Gui::lock();

	if (current >= nb_step) {
		current = nb_step;
		if (current >= ma_step) {
			ma_step += 16;
			step = (Step**) realloc(step, sizeof(Step*) * ma_step);
		}
		nb_step++;
		step[current] = new Step(t);	
	} else if (step[current]->time != t) {
		while (current < nb_step - 1 && step[current]->time < t) current++;
		while (current > 0 && step[current]->time > t) current--;

		if (nb_step >= ma_step) {
			ma_step += 16;
			step = (Step**) realloc(step, sizeof(Step*) * ma_step);
		}
		if (step[current]->time < t) {
			current++;
		}
		for (i = nb_step; i > current;i--) {
			step[i] = step[i - 1]; 
		}

		nb_step++;
		step[current] = new Step(t);		
	}	 
	
	// find the next value for the device channel
	for (i = current + 1; i < nb_step && nt == -1; i++) {
		int j;
		Step *s = step[i];
		for (j = 0; j < s->nb_action; j++) {
			if (s->action[j].device == de && 
				s->action[j].channel == ch) 
			{
				nt = s->time;
				nv = s->action[j].value;
				break;
			}
		}
	}
	for (i = 0; i < current && nt == -1; i++) {
		int j;
		Step *s = step[i];
		for (j = 0; j < s->nb_action; j++) {
			if (s->action[j].device == de && 
				s->action[j].channel == ch) 
			{
				nt = s->time;
				nv = s->action[j].value;
				break;
			}
		}
	}
	
	step[current]->add_action(de, ch, cu, mi, va, nt, nv);

	// find the previous action to set it up
	nt = -1;
	for (i = current - 1; i >= 0 && nt == -1; i--) {
		int j;
		Step *s = step[i];
		for (j = 0; j < s->nb_action; j++) {
			if (s->action[j].device == de && 
				s->action[j].channel == ch) 
			{
				nt = 0;
				s->action[j].next_value = va;
				s->action[j].next_time = t;
				break;
			}
		}		
	}
	for (i = nb_step - 1; i > current && nt == -1; i--) {
		int j;
		Step *s = step[i];
		for (j = 0; j < s->nb_action; j++) {
			if (s->action[j].device == de && 
				s->action[j].channel == ch) 
			{
				nt = 0;
				s->action[j].next_value = va;
				s->action[j].next_time = t;
				break;
			}
		}		
	}
	
	changed = 1;
	Gui::unlock();
}

void StepGroup::update_stack(Step *st)
{
	int i;
	
	Gui::lock();
	
	for (i = 0; i < st->nb_action; i++) {
		int j;
		Action *a;

		a = st->action + i;
		if (nb_stack >= ma_stack) {
			ma_stack += 16;
			stack = (Action*) realloc(stack, sizeof(Action) * ma_stack);
		}
		for (j = 0; j < nb_stack; j++) {
			if (stack[j].device == a->device &&
				stack[j].channel == a->channel)
			{
				memcpy(&stack[j], a, sizeof(Action));
/*
				stack[j].time = a->time;
				//stack[j].channel = a->channel;
				//stack[j].device = a->device;
				stack[j].curve = a->curve;
				stack[j].value = a->value;
				stack[j].next_time = a->next_time;
				stack[j].next_value = a->next_value;
*/
				break;
			}
		}
		if (j == nb_stack) {
			memcpy(&stack[j], a, sizeof(Action));
/*
			stack[j].time = a->time;
			stack[j].channel = a->channel;
			stack[j].device = a->device;
			stack[j].curve = a->curve;
			stack[j].value = a->value;
			stack[j].next_time = a->next_time;
			stack[j].next_value = a->next_value;
*/
			nb_stack++;
		}
	}
	Gui::unlock();
}

void StepGroup::do_step(int t)
{
	int i;

	if (nb_step < 1) return;

	if (current >= nb_step) {
		current = 0;
	}

	if (step[current]->time > t) {
		for (i = current; i < nb_step; i++) {
			update_stack(step[i]);
		}
		current = 0;
	}
	for (i = current; i < nb_step && step[i]->time <= t; i++) {
		update_stack(step[i]);
		current = i;
	}	

	if (!live) return;

	for (i = 0; i < nb_stack; i++) {
		int val, lv;

		switch (stack[i].curve) {
		default:
		case 0: // step curve
			val = stack[i].value;
			break;

		case 1: 
		    { // linear curve
			int dt1, dt2, dt3;
			if (stack[i].time >= t) {
				dt1 = stack[i].time - t;
			} else {
				// the time t is before the current action, 
				// we are in a loop sequence.
				dt1 = time_length - t + stack[i].time;
			}

			if (stack[i].next_time >= t) {
				dt2 = stack[i].next_time - t;
			} else {
				// next action is before the current time
				dt2 = time_length - t + stack[i].next_time; 
			}
			dt3 = dt2 + dt1;
			if (dt3 < 1 || stack[i].next_time < 0) {
				val = stack[i].value;
			} else {
				val = ((dt1 << 8) / dt3) * stack[i].next_value;
				val += ((dt2 << 8) / dt3) * stack[i].value;
				val >>= 8;
			}	
		    }
		}

		switch (stack[i].mix_mode) {
		default:
		case 0: // LTP
			gui->devices->fix[stack[i].device]->
				set_virtual(stack[i].channel, val);
			break;
		case 1: // HTP
			lv = gui->devices->fix[stack[i].device]->
				get_virtual(stack[i].channel);
			if (lv < val || !gui->devices->fix[stack[i].device]->
				has_changed_virtual(stack[i].channel)) 
			{
				gui->devices->fix[stack[i].device]->
					set_virtual(stack[i].channel, val);
			}
			break;
		case 2: // ADD
			if (gui->devices->fix[stack[i].device]->
				has_changed_virtual(stack[i].channel)) 
			{
				val += gui->devices->fix[stack[i].device]->
					get_virtual(stack[i].channel);
			}
			if (val > 255) val = 255;
			gui->devices->fix[stack[i].device]->
				set_virtual(stack[i].channel, val);
			break;
		case 3: // SUB	
			if (gui->devices->fix[stack[i].device]->
				has_changed_virtual(stack[i].channel)) 
			{
				val = gui->devices->fix[stack[i].device]->
					get_virtual(stack[i].channel) - val;
			} else {
				val = 255 - val;
			}
			if (val < 0) val = 0;
			gui->devices->fix[stack[i].device]->
				set_virtual(stack[i].channel, val);
		}
	}
}

void StepGroup::load_word(const char *word, int len, int depth, int closed)
{
	char *b = NULL;
	char bb[256];

	switch (stage1) {
	case -1:
		return;
	case 0:
		if (!strncmp(word, "DmxStep_Step_File", len) && 
			len == (int)strlen("DmxStep_Step_File")) 
		{
			stage1 = 2;
		} else {
			stage1 = -1;
		}
		return;
	case 2:
		if (!strncmp(word, "time_length", len) && len == (int)strlen("time_length")) {
			stage1 = 3;
		}
		return;
	case 3:
		if (closed == 0) {
			stage1 = 20;
			b = (char*) malloc(len + 1);
			strncpy(b, word, len);
			b[len] = 0;
			time_length = atol(b);
			free(b);
		} else {
			stage1 = -1;
		}
		return;
	
	case 20:
		if (!strncmp(word, "bpm", len) && len == 3) {
			stage1 = 21;
		}
		return;
	case 21:
		if (closed == 0) {
			stage1 = 4;
			b = (char*) malloc(len + 1);
			strncpy(b, word, len);
			b[len] = 0;
			bpm = atol(b);
			free(b);
		} else {
			stage1 = -1;
		}
		return;

	case 4:
		if (!strncmp(word, "nb_dev", len) && len == (int)strlen("nb_dev")) {
			stage1 = 5;
		}
		return;
	case 5:
		if (closed == 0) {
			stage1 = 6;
			b = (char*) malloc(len + 1);
			strncpy(b, word, len);
			b[len] = 0;
			nb_dev = atol(b);
			free(b);
			free(dev);
			dev = (int*) malloc(nb_dev * sizeof(int));
		} else {
			stage1 = -1;
		}
		return;

	case 6:
		if (!strncmp(word, "dev_map", len) && len == (int)strlen("dev_map")) {
			stage1 = 7;
			stage2 = 0;
		}
		return;
	case 7:
		if (closed == 0 && stage2 < nb_dev) {
			int i;
			b = (char*) malloc(len + 1);
			strncpy(b, word, len);
			b[len] = 0;
			dev[stage2] = 0;
			for (i = 0; i < gui->devices->nb_fix; i++) {
				if (!strcmp(gui->devices->fix[i]->name, b)) {
					dev[stage2] = i;
					break;
				}
			}	
			stage2++;
			if (stage2 == nb_dev) stage1 = 8;
			free(b);
		} else {
			stage1 = -1;
		}
		return;

	case 8:
		if (!strncmp(word, "step", len) && len == (int)strlen("step")) {
			stage1 = 9;
			stage2 = 0;
			stage_last_level = depth;
		}
		return;
	case 9:
		if (len > 250) {
			stage1 = -1;
			return; 
		}
		if (depth == stage_last_level + 1 && (stage2 == 0 || closed == 2)) {
			stage2 = 1;
			stage3 = 0;
			strncpy(bb, word, len);
			bb[len] = 0;
			stage_time = atol(bb);	
		} else if (depth == stage_last_level + 2 && stage2 == 1) {
			strncpy(bb, word, len);
			bb[len] = 0;
			switch (stage3) {
			case 0:
				stage_device = atol(bb);
				if (stage_device < 0 || stage_device >= nb_dev) {
					stage_device = 0;
				}
				stage_device = dev[stage_device];
				break;
			case 1:
				stage_channel = atol(bb);
				break;
			case 2:
				stage_curve = atol(bb);
				break;
			case 3:
				stage_mix_mode = atol(bb);
				break;
			case 4:
				stage_value = atol(bb);
				add_action(stage_time, stage_device, stage_channel,
					stage_curve, stage_mix_mode, stage_value);
				stage3 = -1; 
			}
			stage3++;
		} else {
			stage1 = -1;
		}
		return;

	}
}

int StepGroup::load(const char *file)
{
	char *buf, *buf1;
	char *ptr1, *ptr2;
	int mlen = 2048;
	int depth = 0;
	int closed = 0;
	FILE *fd;
	int rl;

	if (file) {
		free(file_name);
		file_name = strdup(file);
	}

	if (!file_name) return -1;

	fd = fl_fopen(file_name, "r");
	if (!fd) {
		return -2;
	}

	while (nb_step > 0) {
		nb_step--;
		delete(step[nb_step]);
		step[nb_step] = NULL;;
	}

	nb_stack = 0;

	stage1 = 0;
	stage2 = 0;

	buf = (char*) malloc(mlen);
	buf1 = (char*) malloc(mlen);

	ptr1 = buf;
	while (!feof(fd)) {
		char *end;
		
		if (mlen < ptr1 - buf + 512) {
			mlen += 1024;
			buf = (char*)realloc(buf, mlen);
			buf1 = (char*)realloc(buf1, mlen);
		}
		ptr1 = buf;
		rl = fread(ptr1, 1, mlen - (ptr1 - buf), fd);
		ptr2 = ptr1 = buf;
		end = ptr1 + rl;
		
		while (ptr1 < end) {
			while (ptr1 < end && isspace(ptr1[0])) ptr1++;
			if (ptr1 < end) ptr2 = ptr1;
			if (ptr1 < end && ptr1[0] == '(') { 
				depth++;
				ptr1++;
				ptr2 = ptr1;
			} else if (ptr1 < end && ptr1[0] == ')') {
				depth--;
				ptr1++;
				ptr2 = ptr1;
				closed++;
			} else if (ptr1 < end && ptr1[0] == '"') {
				ptr1++;
				while (ptr1 < end && ptr1[0] != '"') ptr1++;
				if (ptr1 < end) {
					int l;
					ptr1++;
					l = sexpr_unquote(ptr2, ptr1 - ptr2,
							buf1, mlen);
					load_word(buf1, l, depth, closed);
					closed = 0;
					ptr2 = ptr1;
				}		
			} else if (ptr1 < end) {
				while (ptr1 < end && 
					!isspace(ptr1[0]) && ptr1[0] != '(' &&
					ptr1[0] != ')') ptr1++;
				if (ptr1 < end) {
					load_word(ptr2, ptr1 - ptr2, depth, closed);
					closed = 0;
					ptr2 = ptr1;
				}		
			}
		}

		memmove(buf, ptr2, rl - (ptr2 - buf));
	}

	free(buf);
	free(buf1);
	fclose(fd);

	divisor = bpm / 6000.0;
	if (divisor == 0.0) divisor = 1.0;

	changed = 0;

	return 0;
}

int StepGroup::save(const char *file)
{
	int i;
	FILE *fd;

	// fill the stack to get all the devices
	for (i = 0; i < nb_step; i++) {
		update_stack(step[i]);	
	}
	
	for (i = 0; nb_step > current && i <= current; i++) {
		update_stack(step[i]);	
	}

	if (file) {
		free(file_name);
		file_name = strdup(file);
	}

	if (!file_name) return -1;
	
	fl_make_path_for_file(file_name);

	fd = fl_fopen(file_name, "w");
	if (!fd) {
		return -2;
	}

	fprintf(fd, "(DmxStep_Step_File\n");
	fprintf(fd, "  (version  1.0.0)\n");
	fprintf(fd, "  (time_length  %d)\n", time_length);
	fprintf(fd, "  (bpm  %d)\n", bpm);

	fprintf(fd, "  (nb_dev  %d)\n", nb_stack);
	fprintf(fd, "  (dev_map\n");
	for (i = 0; i < nb_stack; i++) {
		char buf[2048];
		const char *name;

		name = gui->devices->fix[stack[i].device]->name;
		sexpr_quote(name, strlen(name), buf, 2048);
		fprintf(fd, "    %s\n", buf);	
	}
	fprintf(fd, "  )\n");

	fprintf(fd, "  (step\n");
	for (i = 0; i < nb_step; i++) {
		int j;
		fprintf(fd, "    (%d\n", step[i]->time);
		for (j = 0; j < step[i]->nb_action; j++) {
			int k;
			int d = step[i]->action[j].device;
			for (k = 0; k < nb_stack; k++) {
				if (d == stack[k].device) {
					d = k;
					break;
				}
			}
			fprintf(fd, "      (%d %d %d %d %d)\n",
				d,
				step[i]->action[j].channel,
				step[i]->action[j].curve,
				step[i]->action[j].mix_mode,
				step[i]->action[j].value);
		}
		fprintf(fd, "    )\n");	
	}
	fprintf(fd, "  )\n");

	fprintf(fd, ")\n");

	fclose(fd);

	changed = 0;

	divisor = bpm / 6000.0;
	if (divisor == 0.0) divisor = 1.0;

	return 0;
}

//
// End of "$Id: $".
//

