//
// "$Id: $"
//
//
//                   Copyright (c) 2003, O'ksi'D
//                      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of O'ksi'D nor the names of its contributors
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//

#include "Universe.h"
#include "Gui.h"
#include <FL/Fl_Window.H>
#include <FL/Fl.H>
#include <FL/fl_draw.H>
#include <FL/fl_ask.H>
#include <FL/Fl_Preferences.H>
#include <stdlib.h>
#include <stdio.h>

#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#ifdef WIN32
#define snprintf _snprintf
#endif

static Fl_Menu_Item ctrl_type_item[] = {
	{"None"},
	{"OksiD DMX 3/1"},
	{"OksiD DMX LC USB"},
	{"OksiD DMX HE USB"},
	{"OksiD DMX HE LPT"},
	{0}
};

static Fl_Menu_Item ctrl_id_lpt_item[] = {
	{"LPT1", 0, 0, (void*)0x378},
	{"LPT2", 0, 0, (void*)0x278},
	{0}
};

static Fl_Menu_Item ctrl_universe_1_item[] = {
	{"1"},
	{0}
};

static Fl_Menu_Item ctrl_universe_3_item[] = {
	{"1"},
	{"2"},
	{"3"},	
	{0}
};

static Fl_Menu_Item ctrl_universe_32_item[] = {
	{"1"},
	{"2"},
	{"3"},	
	{"4"},
	{"5"},
	{"6"},	
	{"7"},
	{"8"},
	{"9"},	
	{"10"},
	{"11"},
	{"12"},	
	{"13"},
	{"14"},
	{"15"},	
	{"16"},
	{"17"},
	{"18"},	
	{"19"},
	{"20"},
	{"21"},	
	{"22"},	
	{"23"},
	{"24"},
	{"25"},	
	{"26"},
	{"27"},
	{"28"},	
	{"29"},
	{"30"},
	{"31"},	
	{"32"},	
	{0}
};

/*
static void check_cb(void *d)
{
	Universe *u = (Universe*) d;
	if (u->shown()) u->refresh();
	Fl::add_timeout(0.5, check_cb, d);
}
*/

static void scroll_in_cb(Fl_Widget *w, void *d)
{
	int i;
	Universe *u = (Universe*) d;
	for (i = 0; i < 511; i++) if (u->scroll_in[i] == w) break;
	Fl_Int_Input *in = u->scroll_in[i];
	u->set_channel(i, atol(in->value()));
}

static void ctrl_cb(Fl_Widget *w, void *d)
{
	Universe *u = (Universe*) d;
	printf("ctrl_cb\n");
	u->scan_controller();
	printf("ctrl_cb out\n");
}

Universe::Universe(int W, int H, int I, int D) :
	Fl_Window(W, H)
{
	char buf[20];
	int i;
	int YY, XX;
	int X, Y;
	X = Y = 0;

	label("Universe Monitor");


	id = I;
	is_output = D;

	current_buffer = 0;

	size_range(W, 80);

	snprintf(buf, 19, "U%s%02d", is_output ? "out" : "inp", id + 1);
	name = new Fl_Box(X + 5, Y + 5, 60, 25, strdup(buf));
	name->labelfont(FL_HELVETICA_BOLD);

	res = new Fl_Box( X + W - 7, Y + 100, 2, 2);
	res->hide();

	ctrl_id_usb_item = (Fl_Menu_Item*) malloc(sizeof(Fl_Menu_Item));
	ctrl_id_usb_item[0].text = 0; //user_data_

	ctrl_type = new Fl_Choice(X + 160, Y + 5, 170, 25, "Control. Type");
	ctrl_type->menu(ctrl_type_item);
	ctrl_type->callback(ctrl_cb, this);
	ctrl_id = new Fl_Choice(X + 395, Y + 5, 70, 25, "Port / ID");
	ctrl_id->menu(ctrl_id_lpt_item);
	ctrl_id->callback(ctrl_cb, this);
	ctrl_universe = new Fl_Choice(X + 532, Y + 5, 50, 25, "Universe");
	ctrl_universe->menu(is_output ? 
		ctrl_universe_3_item : ctrl_universe_1_item);
	ctrl_universe->callback(ctrl_cb, this);


	status = new Fl_Output(X + 52, Y + 32, W - 80, 22, "Status:" );
	status->value("Not connected !!!");

	scroll = new Fl_Scroll( X + 5, Y + 75, W - 10,  H - 80 );
	resizable(res);

	scroll->box(FL_DOWN_BOX);
	YY = 5;
	XX = 5;
	for (i = 0; i < 512; i++) {
		

		snprintf(buf, 19, "%03d", i + 1);
		if (XX + 60 + 16 > scroll->w()) {
			XX = 5;
			YY += 25;
		}
		scroll_in[i] = new Fl_Int_Input(X + 5 + XX, 
					Y + 30 + YY, 30, 20); 
		scroll_in[i]->label(strdup(buf));
		scroll_in[i]->value("000");
		//scroll_in[i]->when(FL_WHEN_NOT_CHANGED|FL_WHEN_CHANGED);
		scroll_in[i]->callback(scroll_in_cb, this);
		scroll_in[i]->box(FL_FLAT_BOX);
		XX += 60;
	}

	blackout();

	end();
	scroll->position(0,0);

	//Fl::add_timeout(1.0, check_cb, this);
}

Universe::~Universe()
{
	int i;
	for (i = 0; i < 512; i++) {
		free((char*)scroll_in[i]->label());
	}
	free((char*)name->label());
	free(ctrl_id_usb_item);
}

void Universe::scan_controller()
{
	status->value("Data transmission...");
	if (ctrl_type->value() == 1 || ctrl_type->value() == 2) {
		int id = (int)ctrl_id->menu()[ctrl_id->value()].user_data_;
		//free(ctrl_id_usb_item);
		//ctrl_id_usb_item = DevIo::create_usb_menu(ctrl_type->value());
		
		//ctrl_id_usb_item = (Fl_Menu_Item*) malloc(sizeof(Fl_Menu_Item));
		//ctrl_id_usb_item[0].text = 0; //user_data_
	}
	DevIo::update_controllers();
}

void Universe::draw()
{
	int X, Y, i;
	X = 5;
	Y = scroll->y() - 3;
	Fl_Window::draw();
	fl_color(FL_WHITE);
	fl_rectf(X, Y - 15, 512, 16);
	fl_color(FL_BLUE);
	for (i = 0; i < 512; i++) {
		fl_line(X + i, Y - (buffer[current_buffer][i] >> 4), X + i, Y);	
	}
}

void Universe::resize(int X, int Y, int W, int H)
{
	int i, XX, YY;
	Fl_Window::resize(X, Y, W, H);
	X = 0;
	Y = 0;
	YY = scroll->y() - 30;
	XX = 35;
	for (i = 0; i < 512; i++) {
		if (XX + 35 + 16 > scroll->w()) {
			XX = 35;
			YY += 25;
		}
		scroll_in[i]->resize(X + 5 + XX, Y + 30 + YY,
			scroll_in[i]->w(), scroll_in[i]->h());
		XX += 60;
	}
	
}

void Universe::blackout(void)
{
	int i;
	Gui::lock();
	for (i = 0; i < 512; i++) {
		buffer[0][i] = 0;
		buffer[1][i] = 0;
		flags[0][i] = 3;
		flags[1][i] = 3;
	}
	Gui::unlock();
}

void Universe::refresh(void)
{
	int c = 0;
	if (!shown()) return;
	for (int i = 0; i < 512; i++) {
		if (flags[current_buffer][i] & 2) {
			char buf[25];
			snprintf(buf, 20, "%03d", buffer[current_buffer][i]); 
			scroll_in[i]->value(buf);	
			c = 1;
			flags[current_buffer][i] &= ~2;
		} 
	}
	if (c) damage(FL_DAMAGE_CHILD);
}


void Universe::swap_buffer()
{
	
	current_buffer = !current_buffer;
	if (is_output) {
		int i;
		for (i = 0; i < 512; i++) {
			buffer[current_buffer][i] = buffer[!current_buffer][i];
			flags[current_buffer][i] = flags[!current_buffer][i] & ~1;
		}
	} else {
		int i;
		for (i = 0; i < 512; i++) {
			if (buffer[current_buffer][i] != 
				buffer[!current_buffer][i]) 
			{
				flags[current_buffer][i] = 
					flags[!current_buffer][i] | 3;
			}
		}
	}
}



//
// End of "$Id: $".
//

