//
// "$Id: $"
//
//
//                   Copyright (c) 2003, O'ksi'D
//                      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of O'ksi'D nor the names of its contributors
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//

#ifndef Universe_H
#define Universe_H

#if defined(WIN32)
#include <windows.h>
#include <process.h>
#else
#include <pthread.h>
#endif

#include <FL/Fl_Window.H>
#include <FL/Fl_Choice.H>
#include <FL/Fl_Slider.H>
#include <FL/Fl_Scroll.H>
#include <FL/Fl_Output.H>
#include <FL/Fl_Box.H>
#include <FL/Fl_Pack.H>
#include <FL/Fl_Int_Input.H>
#include <FL/Fl_Toggle_Button.H>


class Universe : public Fl_Window {
public:
	int is_output;
	int id;

	char current_buffer;
	unsigned char buffer[2][512];
	unsigned char flags[2][512];

	Fl_Scroll *scroll;
	Fl_Int_Input *scroll_in[512];

	Fl_Menu_Item *ctrl_id_usb_item;
	Fl_Choice *ctrl_type;
	Fl_Choice *ctrl_id;
	Fl_Choice *ctrl_universe;

	Fl_Output *status;
	Fl_Box *name;
	Fl_Box *res;

	Universe(int W, int H, int I, int D);
	~Universe();

	void blackout(void);
	void resize(int X, int Y, int W, int H);
	void swap_buffer(void);
	void refresh(void);
	void draw(void);

	void scan_controller(void);

	void set_channel(int channel, int value) {
		buffer[current_buffer][channel] = (uchar) value;
		flags[current_buffer][channel] |= 3;
	}

	int get_channel(int channel) {
		int r;
		flags[current_buffer][channel] &= ~1;
		r = (int)buffer[current_buffer][channel];
		return r;
	}

	int get_nf_channel(int channel) {
		return (int)buffer[current_buffer][channel];
	}
	
	int has_changed(int channel) {
		return (int) flags[current_buffer][channel] & 1;
	}

};


#endif // Universe_H

//
// End of "$Id: $".
//

