//
// "$Id: $"
//
//
//                   Copyright (c) 2003, O'ksi'D
//                      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of O'ksi'D nor the names of its contributors
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//

#include "Universes.h"
#include "DevIo.h"
#include <stdlib.h>
#include <stdio.h>

#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#ifdef WIN32
#define snprintf _snprintf
#define strdup _strdup
#endif

static void but_cb(Fl_Widget *w, void *d)
{
	int i;

	Universes *u = (Universes*) d;
	if (w == u->refresh) {
		DevIo::update_controllers();
		return;
	}

	for (i = 0; i < 32; i++) {
		if (w == u->but_in[i]) {
			Universe *t = ((Gui*)u->parent())->in_universes[i];
			if (t->shown()) {
				t->hide();
			} else {
				t->show();
			}
			return;
		} else if (w == u->but_out[i]) {
			Universe *t = ((Gui*)u->parent())->out_universes[i];
			if (t->shown()) {
				t->hide();
			} else {
				t->show();
			}
			return;
		}
	}
}

Universes::Universes(int X, int Y, int W, int H, const char *T) :
	Fl_Group(X, Y, W, H, T)
{
	int i;

	timeout.tv_sec = 0;
	timeout.tv_usec = 22727; // 44Hz

	scroll = new Fl_Scroll(X + 0, Y + 0, W - 0, H - 0);
	scroll->box(FL_DOWN_BOX);

	for (i = 0; i < 32; i++) {
		char buf[128];
		sprintf(buf, "Uinp%02d", i + 1);
		but_in[i] = new Fl_Button(X + 10, Y + 10 + i * 25, 70, 22, strdup(buf));
		but_in[i]->callback(but_cb, this);
		sprintf(buf, "Uout%02d", i + 1);
		but_out[i] = new Fl_Button(X + 90, Y + 10 + i * 25, 70, 22, strdup(buf));
		but_out[i]->callback(but_cb, this);
	}
	refresh = new Fl_Button(X + 210, Y + 50, 270, 22, "Refresh Controller Connection");
	refresh->callback(but_cb, this);	
	scroll->end();
	hide();
	end();
}

Universes::~Universes()
{
	int i;
	for (i = 0; i < 32; i++) {
		free((char*)but_in[i]->label());
		free((char*)but_out[i]->label());
	}
	
}

//
// End of "$Id: $".
//

