//
// "$Id: $"
//
//
//                   Copyright (c) 2003-2004, O'ksi'D
//                      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of O'ksi'D nor the names of its contributors
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//

#include "sexpr.h"

#include <FL/fl_utf8.H>

int sexpr_quote(const char *in, int in_len, char *buffer, int buffer_len)
{
	int i = 0;
	int oi = 0;
	int lo = 0;

	if (!buffer) buffer_len = 0;
	if (!in) in_len = 0;

	if (oi < buffer_len) buffer[oi++] = '"';

	while (i < in_len) {
		if ((oi - lo) >= 68 && (oi + 1) < buffer_len) {
			buffer[oi++] = '\\'; 
			buffer[oi++] = '\n'; 
			lo = oi;
		}
		switch (in[i]) {
		case '\b':
			if (oi + 1 < buffer_len) {
				buffer[oi++] = '\\';
				buffer[oi++] = 'b';
			} else {
				oi += 2;
			}
			break;
		case '\t':
			if (oi + 1 < buffer_len) {
				buffer[oi++] = '\\';
				buffer[oi++] = 't';
			} else {
				oi += 2;
			}
			break;
		case '\v':
			if (oi + 1 < buffer_len) {
				buffer[oi++] = '\\';
				buffer[oi++] = 'v';
			} else {
				oi += 2;
			}
			break;
		case '\n':
			if (oi + 1 < buffer_len) {
				buffer[oi++] = '\\';
				buffer[oi++] = 'n';
			} else {
				oi += 2;
			}
			break;
		case '\f':
			if (oi + 1 < buffer_len) {
				buffer[oi++] = '\\';
				buffer[oi++] = 'f';
			} else {
				oi += 2;
			}
			break;
		case '\r':
			if (oi + 1 < buffer_len) {
				buffer[oi++] = '\\';
				buffer[oi++] = 'r';
			} else {
				oi += 2;
			}
			break;
		case '"':
			if (oi + 1 < buffer_len) {
				buffer[oi++] = '\\';
				buffer[oi++] = '"';
			} else {
				oi += 2;
			}
			break;
		case '\'':
			if (oi + 1 < buffer_len) {
				buffer[oi++] = '\\';
				buffer[oi++] = '\'';
			} else {
				oi += 2;
			}
			break;
		case '\\':
			if (oi + 1 < buffer_len) {
				buffer[oi++] = '\\';
				buffer[oi++] = '\\';
			} else {
				oi += 2;
			}
			break;
			
		default:
			if (in[i] >= 0x20 && in[i] < 0x7F) {
				if (oi < buffer_len) buffer[oi] = in[i]; 
				oi++;
			} else {
				if (oi + 3 < buffer_len) {
					char *hex = "0123456789abcdef";
					buffer[oi++] = '\\';
					buffer[oi++] = 'x';
					buffer[oi++] = hex[(in[i] >> 4) & 0x0F];
					buffer[oi++] = hex[in[i] & 0x0F];
				} else {
					oi += 4;
				}
			}
			break;
		}

		i++;
	}

	if (oi < buffer_len) buffer[oi++] = '"';
	if (oi < buffer_len) buffer[oi] = 0;
	else return -(oi+1);

	return oi;
}

int sexpr_unquote(const char *in, int in_len, char *buffer, int buffer_len)
{
	int i = 1;
	int oi = 0;

	if (!buffer) buffer_len = 0;
	if (!in) in_len = 0;

	if (in_len < 3) return 0;
	if (in[0] != '"') {
		if (oi < buffer_len) buffer[oi] = 0;
		return 0;
	}

	while (i < in_len) {
		if (in[i] == '\\') {
			int c;
			i++;
			if (i >= in_len) {
				if (0 < buffer_len) buffer[0] = 0;
				return 0;
			}
			switch (in[i]) {
			case 'b':
				if (oi < buffer_len) buffer[oi] = '\b';
				oi++;
				break;
			case 't':
				if (oi < buffer_len) buffer[oi] = '\t';
				oi++;
				break;
			case 'v':
				if (oi < buffer_len) buffer[oi] = '\v';
				oi++;
				break;
			case 'n':
				if (oi < buffer_len) buffer[oi] = '\n';
				oi++;
				break;
			case 'f':
				if (oi < buffer_len) buffer[oi] = '\f';
				oi++;
				break;
			case 'r':
				if (oi < buffer_len) buffer[oi] = '\r';
				oi++;
				break;
			case '\"':
				if (oi < buffer_len) buffer[oi] = '\"';
				oi++;
				break;
			case '\'':
				if (oi < buffer_len) buffer[oi] = '\'';
				oi++;
				break;
			case '\\':
				if (oi < buffer_len) buffer[oi] = '\\';
				oi++;
				break;
			case '\n':
				if (i + 1 < in_len && in[i] == '\r') {
					i++;
				}
				break;
			case '\r':
				if (i + 1 < in_len && in[i] == '\n') {
					i++;
				}
				break;
			case 'x':
				if (i + 2 >= in_len) {
					if (0 < buffer_len) buffer[0] = 0;
					return 0;
				}
				i++;			
				c = in[i] - '0';
				if (c > 9) c = in[i] - 'A' + 10;
				if (c > 0xF) c = in[i] - 'a' + 10;
				if (oi < buffer_len) 
					buffer[oi] = (c << 4) & 0xF0;
				i++;
				c = in[i] - '0';
				if (c > 9) c = in[i] - 'A' + 10;
				if (c > 0xF) c = in[i] - 'a' + 10;
				if (oi < buffer_len) 
					buffer[oi] += c  & 0xF;
				oi++;
				break;
			default:
				if (isdigit(in[i])) {
					if (i + 2 >= in_len) {
						if (0 < buffer_len) 
							buffer[0] = 0;
						return 0;
					}
					c = in[i] - '0';
					if (oi < buffer_len) 
						buffer[oi] = ((c & 0x7) << 6); 
					i++;			
					c = in[i] - '0';
					if (oi < buffer_len) 
						buffer[oi] += ((c & 0x7) << 3); 
					i++;			
					c = in[i] - '0';
					if (oi < buffer_len) 
						buffer[oi] += (c & 0x7);  
					oi++;
				} else {
					if (oi + 1 < buffer_len) {
						buffer[oi++] = '\\';
						buffer[oi++] = in[i];
					} else {
						oi += 2;
					}
				}
				break;
			}
			i++;
		} else if (in[i] == '"') {
			break;
		} else {
			if (oi < buffer_len) buffer[oi] = in[i];
			oi++;
			i++;
		}
	}

	if (oi < buffer_len) buffer[oi] = 0;
	else return -(oi+1);

	if (in[i] != '"') {
		if (0 < buffer_len) buffer[0] = 0;
		return 0;
	}

	return oi;
}

SEXPR *sexpr_open(const char *file_name)
{

	SEXPR *s;
	FILE *fd;

	if (!file_name) return NULL;

	fd = fl_fopen(file_name, "r");
	if (!fd) {
		return NULL;
	}
	
	s = (SEXPR*) malloc(sizeof(SEXPR));
	
	s->fd = fd;
	s->mlen = 1024;
	s->depth = 0;
	s->closed = 0;
	s->melem = 8;
	s->elem = (int*) malloc(s->melem * sizeof(int));
	s->elem[0] = 0;
	s->text = (char*) malloc(s->mlen + 1);
	s->buf = (char*) malloc(s->mlen + 1);
	s->text_len = 0;
	s->ptr1 = s->buf;
	s->ptr2 = s->buf;
	s->end = s->buf;
	s->read_len = 0;
	s->state = 0;
	return s;
}

int sexpr_delete(SEXPR **p)
{
	SEXPR *s;
	if (!p) return -1;
	s = *p;
	if (!s || !s->fd) return -1;
	fclose(s->fd);
	s->fd = NULL;
	free(s->text);
	free(s->buf);
	free(s->elem);
	free(s);
	*p = NULL;
	return 0;
}

int sexpr_read(SEXPR *s)
{
    if (!s) return -1;

    while (1) {
	int off;
	switch (s->state) {

	// fill the buffer
	case 0: 	
		if (feof(s->fd)) return 1;
		off = s->ptr1 - s->buf;
		if (s->mlen < off + 512) {
			s->mlen += 1024;
			s->buf = (char*)realloc(s->buf, s->mlen + 1);
			s->text = (char*)realloc(s->text, s->mlen + 1);
			s->ptr1 = s->buf + off;
		}
		
		s->read_len = fread(s->ptr1, 1, s->mlen - off, s->fd) + off;
		s->ptr2 = s->ptr1 = s->buf;
		s->end = s->ptr1 + s->read_len;
		s->state = 1;
		break;

	// find the begining of a word
	case 1:
		while (s->ptr1 < s->end && isspace(s->ptr1[0])) s->ptr1++;
		if (s->ptr1 >= s->end) {
			int l = s->read_len - (s->ptr2 - s->buf);
			memmove(s->buf, s->ptr2, l);
			s->ptr1 = s->buf + l;
			s->state = 0;
			break;
		}

		s->ptr2 = s->ptr1;
		if (s->ptr1[0] == '(') { 
			s->depth++;
			if (s->depth >= s->melem) {
				s->melem = s->depth + 16;
				s->elem = (int*) realloc(s->elem,
					s->melem * sizeof(int));
			}
			s->elem[s->depth] = 0;
			s->ptr1++;
			s->ptr2 = s->ptr1;
		} else if (s->ptr1[0] == ')') {
			s->depth--;
			if (s->depth < 0) return -2;
			s->elem[s->depth]++;
			s->ptr1++;
			s->ptr2 = s->ptr1;
			s->closed++;
		} else if (s->ptr1[0] == '"') {
			s->ptr1++;
			s->state = 2;
		} else  {
			s->state = 3;	
		}
		break;

	// unquote word
	case 2:
		while (s->ptr1 < s->end && s->ptr1[0] != '"') s->ptr1++;
		if (s->ptr1 >= s->end) {
			int l = s->read_len - (s->ptr2 - s->buf);
			memmove(s->buf, s->ptr2, l);
			s->ptr1 = s->buf + l;
			s->state = 0;
			break;
		}
		
		s->ptr1++;
		s->text_len = sexpr_unquote(s->ptr2, s->ptr1 - s->ptr2,
						s->text, s->mlen);
		if (s->text_len < 0) s->text_len = 0;
		s->text[s->text_len] = 0;
		s->state = 4;
		return 0;

	// simple word -> copy it
	case 3:
		while (s->ptr1 < s->end && 
			!isspace(s->ptr1[0]) && s->ptr1[0] != '(' &&
			s->ptr1[0] != ')') 
		{
			s->ptr1++;
		}
		if (s->ptr1 >= s->end) {
			int l = s->read_len - (s->ptr2 - s->buf);
			memmove(s->buf, s->ptr2, l);
			s->buf[l] = 0;
			s->ptr1 = s->buf + l;
			s->state = 0;
			break;
		}
		s->text_len = s->ptr1 - s->ptr2;
		memcpy(s->text, s->ptr2, s->text_len);
		s->text[s->text_len] = 0;
		s->state = 4;
		return 0;

	// the word has been processed !
	case 4:
		s->elem[s->depth]++;
		s->closed = 0;
		s->ptr2 = s->ptr1;
		s->state = 1;
		break;
	}
    }
}

//
// End of "$Id: $".
//

